
; Returns vector with mag data or -1 on error
;
function thm_part_slice2d_getmag, laststruct, datstructs, mag_data=mag_data, $
                                rotation=rotation, fail=fail

    compile_opt idl2, hidden
  
  if keyword_set(mag_data) then begin
    ; user specified tplot variable 
    bfield = dat_avg(mag_data, datStructs[0].time, laststruct.end_time, /interp)
  endif else begin
    ; data added in thm_part_dist_array
    bfield = average(datStructs.magf, 2)
    
    ; check for NaNs that could corrupt rotation matrix
    if in_set(finite(bfield),0) then begin
      if ~in_set(['xy','xz','yz','xvel'], rotation) then begin
        fail = 'Magnetic field data incomplete for desired time range.'+ $
               ' Use the MAG_DATA keyword to specify a tplot variable.'
        dprint, fail
        return, -1
      endif
    endif 
    
  endelse
  
    return, bfield

end



;Returns bulk velocity vector or -1 on error
;
;Note: thm_esa_slice2d Divides total flux (over entire time range) by 
;   the total density; here the flux/density was previously 
;   calculated for each distribution and is now averaged.
function thm_part_slice2d_getvel, thedata, datstructs, vel_data=vel_data, $
                                rotation=rotation, fail=fail

    compile_opt idl2, hidden
  
  if keyword_set(vel_data) then begin
    ; user specified variable
    vbulk = dat_avg(vel_data, datStructs[0].time, thedata.end_time, /interp)
  endif else begin
    ; data added in thm_part_dist_array
    vbulk = average(datStructs.velocity, 2) ;vel in km/s
    
    ; Check for NaNs that could corrupt rotation matrix
    if in_set(finite(vbulk), 0) then begin
      if in_set(['BV','BE','xvel','perp'], rotation) then begin 
        fail='Bulk velocity data incomplete for desired time range.'+ $
             '  Use the VEL_DATA keyword to specify a tplot variable.'
        dprint, fail
        return, -1
      endif
    endif
  
  endelse

  ; convert to m/s
  return, vbulk * 1000.

end


;Performs rotations from DSL to requested coordinates (COORD keyword)
; *Assumes the transformation from the data's native DSL to
;  the requested coords (GSM, GSE) is invariant over the
;  time range of the slice.* 
pro thm_part_slice2d_cotrans, vectors, bfield, vbulk, fail=fail, $
                              probe=probe, coord=coord, twin=twin, matrix=matrix

    compile_opt idl2, hidden


  ctn = '2dslice_temp_cotrans'
  if stregex(coord, '^dsl$', /bool, /fold_case) then return
  if ~keyword_set(probe) then begin
    fail = 'Cannot determine spacecraft. Coordinate transform canceled.'
    dprint,dlevel=1, fail
    return
  endif
  if ~stregex(coord, '(^gsm$)|(^gse$)', /bool, /fold_case) then begin
    fail = 'Invalid coordinate keyword: '+coord
    dprint,dlevel=1, fail
    return
  endif
  probe = strlowcase(probe)
  coord = strlowcase(coord)

  dprint, dlevel=4, 'Rotating to '+strupcase(coord)+' coordinates'

  ;Load necessary support data (2 min padding at each end)
  thm_load_state, probe=probe, trange = twin + 120*[-1,1], suffix='_'+ctn, $
                  /get_support_data, /keep_spin_data, verbose=0


  ;Transform x,y,z unit vectors to new coordinates
  store_data, ctn, data = {x:replicate(mean(twin),3), $
                           y: [ [1.,0,0], [0,1,0], [0,0,1] ] }, verbose=0
  thm_cotrans, ctn, probe=probe, in_coord='dsl', out_coord=coord, $
               support_suffix='_'+ctn, out_suff='_'+coord


  ;Form transformation matrix from transformed basis
  get_data, ctn+'_'+coord, data=ctd
  if size(ctd,/type) ne 8 then begin
    fail = 'Could not obtain coordinate transform from DSL -> '+strupcase(coord)+ $
           ', STATE data may be absent for the requested period.'
    dprint, fail
    store_data, '*'+ctn+'*', /delete, verbose=0 ;delete temp vars
    return
  endif
  matrix = transpose(ctd.y)


  ;Apply to velocity data and required rotation vectors
  if keyword_set(type) then vectors = matrix ## temporary(vectors)
  if keyword_set(vbulk) then vbulk = matrix ## vbulk
  if keyword_set(bfield) then bfield = matrix ## bfield


  ;Delete temporary tplot data
  store_data, '*'+ctn+'*', /delete, verbose=0

end



;Retrieves rotation matrix for FAC
; *Assumes the transformation is invariant over the slice's time range* 
pro thm_part_slice2d_fac, rotation, probe=probe, mag_data=mag_data, $
                          twin=twin, matrix=matrix, fail=fail

    compile_opt idl2, hidden


  ctn = '2dslice_temp_fac'
  if ~keyword_set(probe) then begin
    fail = 'Cannot determine spacecraft. FAC transform canceled.'
    dprint,dlevel=1, fail
    return
  endif
  if ~keyword_set(tnames(mag_data)) then begin
    fail = 'Cannot perform FAC rotation without B-field data. '+ $
           'Use MAG_DATA keyword to pass in valid tplot variable.'
    dprint, dlevel=1, fail
    return 
  endif

  probe = strlowcase(probe)
  rotation = strlowcase(rotation)

  dprint, dlevel=4, 'Rotating to FAC using '+rotation

  ;Get averaged bfield data
  get_data, mag_data, dlimits = mdl ;will need dlimits later
  bfield = dat_avg(mag_data, twin[0], twin[1], /interp)
  if total(finite(bfield,/nan)) gt 0 then begin
    fail = 'Cannot perform FAC rotation.  The data in '+ $
           mag_data+' is out of range.'
    dprint, dlevel=1, fail
    return
  endif
  
  ;Store averaged bfield data for thm_fac_matrix_make
  store_data, 'th'+probe+ctn+'_bfield', verbose=0, dlimits=mdl, $
               data = {x:mean(twin), y: reform(bfield,1,n_elements(bfield))} 

  ;Load necessary support data (5 min padding at each end)
  thm_load_state, probe=probe, trange = twin + 300*[-1,1], $ ;, suffix='_'+ctn, $
                  /get_support_data, /keep_spin_data, verbose=0


  ;Create rotation matrix
  thm_fac_matrix_make, 'th'+probe+ctn+'_bfield', other_dim=rotation, $
                       pos_var_name='th'+probe+'_state_pos', $ ;_'+ctn, $
                       newname = ctn+'_mat'

  ;Get rotation matrix from tplot variable
  get_data, ctn+'_mat', data=ctd
  if size(ctd,/type) ne 8 then begin
    fail = 'Could not obtain FAC ('+rotation+') transform'+ $
           ', STATE data may be absent for the requested period.'
    dprint, fail
    store_data, '*'+ctn+'*', /delete, verbose=0 ;delete temp vars
    return
  endif
  matrix = transpose(ctd.y)

  ;Delete temporary tplot data
  store_data, '*'+ctn+'*', /delete, verbose=0

end



; Performs rotation from the standard coordinates (set with COORD keyword)
; to the user specified coordinates (set with ROTATION keyword) 
pro thm_part_slice2d_rotate, velocities, bfield, vbulk, fail=fail, $
                             rotation=rotation, mag_data=mag_data, $
                             twin=twin, probe=probe, matrix=matrix 

    compile_opt idl2, hidden

  ; Create/get rotation matrix
  case strlowcase(rotation) of
    'bv': matrix=cal_rot(bfield,vbulk)
    'be': matrix=cal_rot(bfield,crossp(bfield,vbulk))
    'xy': matrix=cal_rot([1,0,0],[0,1,0])
    'xz': matrix=cal_rot([1,0,0],[0,0,1])
    'yz': matrix=cal_rot([0,1,0],[0,0,1])
    'xvel': matrix=cal_rot([1,0,0],vbulk)
    'perp': matrix=cal_rot(crossp(crossp(bfield,vbulk),bfield),crossp(bfield,vbulk))
    'perp_yz': matrix=cal_rot(crossp(crossp(bfield,[0,1,0]),bfield),crossp(crossp(bfield,[0,0,1]),bfield))
    'perp_xy': matrix=cal_rot(crossp(crossp(bfield,[1,0,0]),bfield),crossp(crossp(bfield,[0,1,0]),bfield))
    'perp_xz': matrix=cal_rot(crossp(crossp(bfield,[1,0,0]),bfield),crossp(crossp(bfield,[0,0,1]),bfield))
    else: begin
    
      ;handle FAC rotations through thm_fac_matrix_make
      valid_fac_coords = ['xgse','ygsm','zdsl', 'rgeo','mrgeo', 'phigeo','mphigeo', 'phism','mphism']
      if total(stregex(valid_fac_coords, rotation, /bool, /fold_case)) gt 0 then begin    
        thm_part_slice2d_fac, rotation, probe=probe, mag_data=mag_data, $
                              twin=twin, matrix=matrix, fail=fail                        
        if keyword_set(fail) then return
      endif else begin
        fail = 'Unrecognized rotation: "'+rotation+'".'
        dprint, fail
        return
      endelse
      
    end
  endcase
  
  ; Prevent data from being mutated to doubles
  matrix = float(matrix)

  if rotation ne 'xy' then begin
    dprint, dlevel=4, 'Alligning slice plane to ' +rotation
  endif else return
  
  ; Rotate data
  if keyword_set(type) then velocities = matrix ## temporary(velocities)
  if keyword_set(vbulk) then vbulk = matrix ## vbulk

end


; Perform rotation from user specified coordinates to the 
; slice plane's coordinates (determined by SLICE_NORM and SLICE_X)
pro thm_part_slice2d_orientslice, velocities, vbulk, fail=fail, $
                                  slice_x=slice_x, slice_z=slice_z, matrix = matrix
                                  
    compile_opt idl2, hidden

  tolerance = 5d-5 ;arbitrary

  ;error check/set defaults
  if keyword_set(slice_x) then begin
    if n_elements(slice_x) ne 3 then begin
      fail='Invalid x-axis, please specify a valid 3-vector'  
      return
    endif
  endif else begin
    slice_x = [1d,0,0]
  endelse
  
  if keyword_set(slice_z) then begin
    if n_elements(slice_z) ne 3 then begin
      fail='Invalid slice normal, please specify a valid 3-vector'
      return
    endif
  endif else begin
    slice_z = [0,0,1d]
  endelse
  
  ;copy to new vars
  z = double(slice_z)
  z = z/norm(z)
  xp = double(slice_x)
  xp = xp/norm(xp)
  
  ;get y axis
  y = crossp(z,xp)
  
  if norm(y) lt tolerance then begin
    fail='Cannot orient slice plane, x-axis and slice normal are too close to being parallel.'
    return
  endif

  y = y/norm(y)
  
  ;get x axis
  x = crossp(y,z)
  
  if norm(x) lt tolerance then begin ;this should never be true
    fail='Cannot orient slice plane due to an unkown error.'
    return
  endif
  
  x = x/norm(x)

  ;get rotation matrix
  matrix = float([[x],[y],[z]])
  
  ;rotate data
  if keyword_set(type) then velocities = matrix ## temporary(velocities)
  if keyword_set(vbulk) then vbulk = matrix ## vbulk

end


; Subtracts bulk velocity vector from velocities array
; todo: fix for geo
pro thm_part_slice2d_subtract, velocities, vbulk, vel_data=vel_data

    compile_opt idl2, hidden

  if keyword_set(vel_data) then begin
    dprint, dlevel=4,'Velocity used for subtraction is '+vel_data
  endif else begin
    dprint, dlevel=4, 'Velocity used for subtraction is V_3D'
  endelse

  velocities[*,0] = velocities[*,0] - vbulk[0]
  velocities[*,1] = velocities[*,1] - vbulk[1]
  velocities[*,2] = velocities[*,2] - vbulk[2]
  

end




;+
;Procedure: thm_part_slice2d
;
;Purpose: Returns a 2-D slice of the 3-D THEMIS ESA/SST ion or electron distribution
;         function.  The 2-D slice is returned via the PART_SLICE, XGRID, YGRID, and 
;         SLICE_INFO keywords.
;         
;         This procedure works in conjunction with thm_part_dist_array.pro and 
;         plot_part_slice2d.pro.  The corresponding crib is thm_crib_part_slice2d.pro 
;
;   There are currently three methods of generating slices for plotting:
;
;   Geomtric:
;     Exact method showing the bounds of all bins intersecting the slice plane.
;     Values are averaged in cases of overlapping bins or when the slice plane 
;     is coplanar with a bin's boundary.  
;
;   2D Interpolation (from thm_esa_slice2d):
;     Datapoints within the specified theta or z-axis range are projected onto 
;     the slice plane and linearly interpolated onto a regular 2d grid. 
;     X,Y-range values determined slightly differently for 2D Interpolation
;
;   3D Interpolation:
;     The entire 3-dimensional distribution is linearly interpolated onto a 
;     regular 3d grid and a this slice is extracted.  The slice will consist of
;     up to resolution^2 datapoints from within 0.866/resolution of the total 
;     velocity range along the normal. This method works best with dense data.
;       Note: - Not all points within the aforemention range will be used.
;             - Interpolation may occur across data gaps or areas with recorded zeroes
;             - Higher resolution regridding will severely slow this method
;     
;     
;    
;Callins Sequence:
;    thm_part_slice2d, datArr, [datArr2, [datArr3, [datArr4]]], $
;                      timewin = timewin, slice_time=slice_time, $
;                      part_slice=part_slice, xgrid=xgrid, ygrid=ygrid, $
;                      slice_info=slice_info, $
;                      fail=fail
;
;
;Arguments:
; DATARR[#]: An array of data structures as returned by one of the get_th?_p???
;            routines (or thm_part_dist_array.pro).  The structure must have
;            a magf field containing a three-component magnetic field vector.
; 
; 
;Input Keywords:

; SLICE_TIME: Beginning of time window in seconds since Jan. 1, 1970.  If
;             CENTER_TIME keyword set, then TIME is the center of the time widow
;             specified by the TIMEWIN keyword.
; TIMEWIN: Length in seconds over which to compute the slice.
; CENTER_TIME: Flag that, when set, centers the time window around the time 
;              specified by SLICE_TIME keyword. 
; 
; COORD: A string designating what coordinate system the slice will be based off.
;        Default is 'DSL', but can also be 'GSE' or 'GSM'
; ROTATION: The rotation keyword further specifies the coordinates system used
;           to create a slice. A description of each new set of cartesian coordinates
;           is given below.  Some rotations are invariant of the coordinates
;           specified by COORD (BV,perp), other are not (xy,perp_xy). By default
;           the slice will be cut along the x-y plane of the given coordinates.
;                      
;           Field alligned coordinates (FAC) are also available.
;           See thm_fac_matrix_make for available options.  Any valid input to
;           the OTHER_DIM keyword may be used.
;           
;   'BV': The x axis is parallel to B field; the bulk velocity defines the x-y plane.
;   'BE': The x axis is parallel to B field; the B x V(bulk) vector defines the x-y plane.
;   'xy'/'xyz': (Default) The x axis is V_x and the y axis is V_y.
;   'xz': The x axis is V_x and the y axis is V_z.
;   'yz': The x axis is V_y and the y axis is V_z.
;   'xvel': The x axis is V_x; the y axis is defined by the bulk velocity. 
;   'perp': The x axis is the bulk velocity projected onto the plane normal to the B field; y is B x V(bulk)
;   'perp_xy'/'perp_xyz': Geometric xy coordinates projected onto the plane normal to the B field.
;   'perp_xz': Geometric xz coordinates projected onto the plane normal to the B field.
;   'perp_yz': Geometric yz coordinates projected onto the plane normal to the B field.
; 
; SLICE_X/SLICE_NORM: These keywords respectively specify the slice plane's 
;               x-axis and normal within the coordinates specified by 
;               COORD and ROTATION. Both keywords take 3-vectors as input.
;               
;               If SLICE_X is not specified then the given coordinate's 
;               x-axis will be used. If SLICE_X is not perpendicular to 
;               the normal it's projection onto the slice plane will be used.
;               An error will be thrown if no projection exists.
;               
;               If SLICE_NORM is not specified then the given coordinate's
;               z-axis will be used (slice along by x-y plane in those 
;               coordinates).
;                 
;       examples:
;         Slice plane perpendicular to DSL z-axis using [3,2,0] as plane's x-axis: 
;         (this is the same as only using SLICE_X=[3,2,1])
;           COORD='dsl' (default), ROTATION='xyz' (default), SLICE_X=[3,2,1]
;         
;         Slice plane perp. to GSE x-axis, bulk velocity used to define plane's x-axis:
;           COORD='gse', ROTATION='xvel', SLICE_NORM=[1,0,0], SLICE_X=[0,1,0]
;
;         Slice plane along the B field and radial position vectors, B field used as slice's x-axis:
;           ROTATION='rgeo', SLICE_NORM=[0,1,0], SLICE_X=[0,0,1]
;
; DISPLACEMENT: Value in m/s that specifies how far from the origin the slice 
;               plane will be cut.
;               
;       example:
;         Slice plane cut at Z_gse = 500
;           COORD='gse', ROTATION='xyz' (default), DISPLACEMENT=500.
;
;
; TWO_D_INTERP: Flag to use 2D interpolation method (described above)
; THREE_D_INTERP: Flag to use 3D interpolation method (described above)
;
;
; UNITS: A string designating the desired units ('Counts','DF','Rate','CRate',
;        'Flux','EFlux','E2Flux','E3Flux')
;        Default = DF
; ONECOUNT: Flag to remove bins that fall below the once count average after averaging.
; RESOLUTION: A single integer specfying the resolution of each dimension of the
;             slice (as well as the interpolated volume in the the 3d case).
;             Default = 250
; ERANGE: Two element array specifying the energy range to be used
; SMOOTH: An odd integer >=3 specifing the width of the smoothing window in # 
;         of points. Even entries will be incrimented, 0 and 1 are ignored.
; 
; 
; REGRID: (2D/3D Interpolation only)
;         A three element array specifying regrid dimensions in phi, theta, and 
;         energy respectively. If set, all distributions' data will first be 
;         spherically interpolated using the nearest neighbor method. For example  
;         the default [64,32,64] will yield distributions with 63x32 points per 
;         energy, and 64 energy bins). Increasing the regridding resolution will
;         slow slice production, particularly if using 3D interpolation.
;         Default = [64,32,64]
; THETARANGE: (2D interpolation only)
;             Angle range, in degrees [-90,90], used to calculate slice.
;             Default = [-20,20]; will override ZDIRRANGE. 
; ZDIRRANGE: (2D interpolation only)
;            Z-Axis range, in km/s, used to calculate slice.
;            Ignored if called with THETARANGE.
;
;
;Output variables for plotting:
; PART_SLICE: The 2-D data array to be plotted (the slice).
; XGRID: Array of x-locations for the slice.
; YGRID: Array of y-locations for the slice.
; SLICE_INFO: Structure to be passed to plot_part_slice2d to determine plot labels.
;      {
;       probe: string containing the probe
;       dist: string (or array of) containing the type of distribution used
;       coord: string describing the coordinate system used for the slice
;       rot: string describing the user specified rotation (N/A for 2D interp)
;       units: string describing the units
;       twin: string describing the time window of the slice
;       range: array containing the range of the un-interpolated data 
;       vrange: array containing the velocity range of the instrument(s)
;       trange: array containing the numerical time range
;       shift: 3-vector containing any translations made in addition to 
;              requested rotations (e.g. subtracted bulk velocity)
;       bulk: 3-vector containing the bulk velocity in the slice plane's 
;             coordinates
;       coord_m: Rotation matrix from original data's coordinates (DSL) to
;                those specified by the COORD keyword.
;       rot_m: Rotation matrix from the the specified coordinates to those 
;            defined by ROTATION.
;       orient_m: Rotation matrix from the coordinates defined by ROTATION to 
;                 the coordinates defined by SLICE_NORM and SLICE_X 
;                 (column matrix of new coord's basis).
;       }
;
;
;CAVEATS: Due to IDL software constraints regions containing no data 
;         are assigned zeros instead of NaNs.  
;
;
;CREATED BY: A. Flores
;            Based on work by Bryan Kerr and Arjun Raj 
;
;EXAMPLES:  see the crib file: thm_crib_part_slice2d.pro
;-

pro thm_part_slice2d, datArray, datArray2, datArray3, datArray4, $
                    ; Time options
                      timewin=timewin, slice_time=slice_time, $
                      center_time=center_time, $
                    ; Range limits
                      erange=erange, $
                      thetarange=thetarange, zdirrange=zdirrange, $
                    ; Orientations
                      coord=coord, rotation=rotation, $
                      slice_x=slice_x, slice_norm=slice_z, $
                      displacement=displacement, $  
                    ; Support Data
                      mag_data=mag_data, vel_data=vel_data, $
                    ; Type options
                      type=type, two_d_interp=two_d_interp, $
                      three_d_interp=three_d_interp, $
                    ; Other options
                      units=units, resolution=resolution, $
                      subtract=subtract,smooth=smooth,onecount=onecount, $
                      regrid=regrid, slice_width=slice_width, $
                    ; Output
                      part_slice=part_slice, xgrid=xgrid, ygrid=ygrid, $
                      slice_info=slice_info, $
                    ; Other
                      _extra = _extra, fail=fail

    compile_opt idl2

if size(datArray,/type) ne 8 then begin
  fail = 'Invalid data structure.  Input must be valid array of '+ $
          'ESA or SST distributions with required tags. '+ $
          'See thm_part_dist_array.'
  dprint, fail
  return
endif

if ~keyword_set(slice_time) then begin
  fail = 'Please specifiy a time at which to compute the slice.'
  dprint, fail
  return
endif

if ~keyword_set(timewin) then begin
  fail = 'Please specifiy a time window for the slice."
  dprint, fail
  return
endif

d_set = [keyword_set(datarray), keyword_set(datarray2), $
          keyword_set(datarray3), keyword_set(datarray4)]

if total(d_set) gt 1 and keyword_set(type) then begin
  fail = 'Multiple distributions are only supported for the '+ $
         'geometric method.'
  dprint, fail
  return
endif



fail = ''


; Defaults
;---------
if ~keyword_set(coord) then coord='dsl'
if ~keyword_set(rotation) then rotation='xy'
if ~keyword_set(units) then units = 'df'
if ~keyword_set(slice_z) then slice_z = [0,0,1.]
if ~keyword_set(displacement) then displacement = 0.
if rotation eq 'xyz' then rotation = 'xy'
if rotation eq 'perp_xyz' then rotation = 'perp_xy'
probe = keyword_set(datArray[0].spacecraft) ? datArray[0].spacecraft : $
                 strmid(datArray[0].project_name, 0, /reverse_offset)
if ~stregex(probe, '[abcde]', /bool, /fold_case) then probe = ''

;Type specific:
if keyword_set(two_d_interp) then type = 2
if keyword_set(three_d_interp) then type = 3
if size(type,/type) eq 0 then type = 0
if type gt 3 then type = 0

; 2d interp
if type eq 2 then begin 
  if size(smooth,/type) eq 0 then smooth = 3
  if size(resolution,/type) eq 0 then resolution = 51
  if ~keyword_set(thetarange) and ~keyword_set(zdirrange) then begin
    thetarange = [-20.,20]
  endif
endif 

; 2d nn
if type eq 1 then begin
  if size(resolution,/type) eq 0 then resolution = 250
  if keyword_set(regrid) && n_elements(regrid) eq 3 then begin
    regrid = float(regrid)
  endif else $
    regrid = [64.,32,64]
endif

; geometric
if ~keyword_set(type) then begin
  ;nothing for now
endif

;resolution
if ~keyword_set(resolution) then begin
  resolution = keyword_set(type) ? 150:500 
endif


dprint, dlevel=2, 'Processing slice at ' + time_string(slice_time, format=5) +'...'


; Account for multiple data types
; -------------------------------
ds = [ptr_new(datArray), ptr_new(datArray2), $
      ptr_new(datArray3), ptr_new(datArray4)]
ds = ds[where(d_set)]



; Get time info
;--------------

; get the boundaries of the time window
if keyword_set(center_time) then begin
  tmin = slice_time - timewin/2
  tmax = slice_time + timewin/2
endif else begin
  tmin = slice_time
  tmax = slice_time + timewin
endelse

; Cut dist. arrays to time limits
for i=0, n_elements(ds)-1 do begin
  ; get indexes of dat structures in reqested time window
  times = (*ds[i]).time + ((*ds[i]).end_time - (*ds[i]).time)/2
  times_ind = where(times ge tmin AND times le tmax, ndat)
  if ndat gt 0 then begin
    *ds[i] = (*ds[i])[times_ind]
  endif else begin
    dprint, dlevel=4, 'No particle distributions exist in data array '+ $
           strtrim(i,2)+' that are within the requested time window.'
    ds[i] = ptr_new() ;no need to free pointer
    continue
  endelse
endfor

; Remove arrays that were out of range
intrange = where(ptr_valid(ds), cin)
if cin gt 0 then begin
  ds = ds[intrange]
endif else begin
  fail = 'There are no distributions within the given time window ('+ $
         time_string(slice_time)+ ' + '+strtrim(timewin,2)+' sec).'
  dprint, fail
  return
endelse



; Produce a concatenated list of cartesian-based
; velocity bins (and associated data points)by 
; looping over data types
; 
; If geometric mode is being used (type=0) then
; dphi, dtheta, and dr arrays are also returned 
;-------------------------------
for i=0, n_elements(ds)-1 do begin

  ;Get cartesian velocities
  thm_part_slice2d_getXYZ, *ds[i], units, type, $
                           onecount=onecount, regrid=regrid, erange=erange, $
                           orange=org, vrange=vrange, $ 
                           velocities=v0, $           ;array of velocities
                           datapoints=dpoints, $      ;arraw of data
                           dp=dp0, dt=dt0, dv=dv0, $  ;dphi, dtheta, dr
                           fail=fail
  if keyword_set(fail) then return

  ;Concatenate velocities from different distributions
  if keyword_set(velocities) then begin
    velocities = [ velocities, temporary(v0)]
    datapoints = [datapoints, temporary(dpoints)]
  endif else begin
    velocities = temporary(v0)
    datapoints = temporary(dpoints)
  endelse

  ;concatenate extra variables for geometric method
  if keyword_set(dp0) then begin
    if keyword_set(dp) then begin
      dp = [ dp, temporary(dp0)]
      dt = [ dt, temporary(dt0)]
      dv = [ dv, temporary(dv0)]
    endif else begin
      dp = dp0
      dt = dt0
      dv = dv0
    endelse
  endif

  orange = keyword_set(orange) ? minmax([orange,org]):org

endfor
  
;pull last struct from the array to use later
last = (*ds[0])[n_elements(*ds[0])-1]


; For testing only: shows 3-D distribution of nonzero data
if keyword_set(vis) then begin
  rr = where(datapoints ne 0)
  rg = [-max(velocities),max(velocities)]
  iplot, velocities[rr,0], velocities[rr,1], velocities[rr,2], linestyle=6, sym_index=1, $
         xrange = rg, yrange=rg, zrange=rg, $
         rgb_table=13, vert_colors=bytscl(alog10(datapoints[rr]))
endif



; Apply coordinate transforms and rotations
; *For geometric mode, transformations will be
;  applied inside thm_part_slice2d_geo instead
;------------------------------------------

; Get mag data
bfield = thm_part_slice2d_getmag(last, *ds[0], mag_data=mag_data, rotation=rotation, fail=fail)
if n_elements(bfield) eq 1 then return


; Get bulk velocity (in m/s)
vbulk = thm_part_slice2d_getvel(last, *ds[0], vel_data=vel_data, rotation=rotation, fail=fail)
if n_elements(vbulk) eq 1 then return


; Subtract bulk velocity vector?
if keyword_set(subtract) and type ne 0 then begin
  thm_part_slice2d_subtract,  velocities, vbulk, vel_data=vel_data
endif


; Rotate to GSM or GSE if requested
thm_part_slice2d_cotrans, velocities, bfield, vbulk, coord=coord, probe=probe, $
                          twin=[tmin,tmax], matrix=ct, fail=fail
if keyword_set(fail) then return


; Rotate data to desired coordinates
thm_part_slice2d_rotate, velocities, bfield, vbulk, rotation=rotation, probe=probe, $
                         mag_data=mag_data, $ ;need var to get dlimits, for now
                         twin=[tmin,tmax], matrix=rot, fail=fail
if keyword_set(fail) then return


; Get transformation into slice plane's coordinates
thm_part_slice2d_orientslice, velocities, vbulk, fail=fail, $
                              slice_x=slice_x, slice_z=slice_z, matrix=mt
if keyword_set(fail) then return 
                                  


; Misc.
;------

; Convert velocities to km/s
factor = 1000.
velocities = temporary(velocities)/factor
vbulk = vbulk/factor
if keyword_set(dv) then dv = dv/factor
if keyword_set(vrange) then vrange = vrange/factor


;  Visualization for testing:
;  rr = where(datapoints ne 0)
;  rg = [-max(velocities),max(velocities)]
;  iplot, velocities[rr,0], velocities[rr,1], velocities[rr,2], linestyle=6, sym_index=1, $
;         xrange = rg, yrange=rg, zrange=rg, $
;         rgb_table=13, vert_colors=bytscl(alog10(datapoints[rr]))


; Sort transformed velocity grid
sorted = sort(velocities[*,0])
velocities = velocities[sorted,*]
if keyword_set(dp) then begin
  dp = dp[sorted]
  dt = dt[sorted]
  dv = dv[sorted]
endif
datapoints = datapoints[sorted]



; Create slice:
; 
; TYPE=0   Geometric Method
; TYPE=1   2D Nearest Neighbor Interpolation
; TYPE=2   2D Linear Interpolation
; TYPE=3   3D Linear Interpolation
;------------------------------------------


; Linear 2D interpolation from thm_esa_slice2D
if type eq 2 then begin

  dprint, dlevel=4, 'Using 2d linear interpolation'
  thm_part_slice2d_2di, datapoints, velocities, resolution, $
                          thetarange=thetarange, zdirrange=zdirrange, $
                          part_slice=part_slice, $
                          xgrid=xgrid, ygrid=ygrid, $
                          fail=fail

; Linear 3D Interpolation
endif else if type eq 3 then begin

  dprint, dlevel=4, 'Using 3d linear interpolation'
  thm_part_slice2d_3di, datapoints, velocities, resolution, $
                          orange=orange, displacement=displacement, $
                          part_slice=part_slice, $
                          xgrid=xgrid, ygrid=ygrid, $
                          fail=fail


; 2D Nearest Neighbor (for testing only)
endif else if type eq 1 then  begin

  dprint, dlevel=4, 'Using 2d nearest neighbor'
  thm_part_slice2d_nn, datapoints, velocities, resolution, slice_orient, $
                          part_slice=part_slice, $
                          xgrid=xgrid, ygrid=ygrid, $
                          slice_width=slice_width, $
                          shift=keyword_set(subtract) ? vbulk:0, $
                          fail=fail

; Geometric Method
endif else begin

  dprint, dlevel=4, 'Using geometric method'
  thm_part_slice2d_geo, datapoints, velocities, resolution, $
                        dp, dt, dv, displacement=displacement, $
                        ct=ct, rot=rot, mt=mt, $
                        part_slice=part_slice, $
                        xgrid=xgrid, ygrid=ygrid, $
                        fail=fail

endelse

if keyword_set(fail) then begin
  dprint, dlevel=1, fail
  return
endif


; Apply smoothing
if keyword_set(smooth) then begin
  smooth = round(smooth)
  if smooth ge 2 then begin 
    part_slice = smooth(part_slice, smooth)
    negidx = where(part_slice lt 0, nneg)
    if nneg gt 0 then begin
      part_slice[negidx] = 0
    endif
  endif else begin
    dprint, dlevel=1, 'Smoothing not applied.  Smoothing value must be >= 2'
  endelse
endif



; Pass out slice information for plotting
;----------------------------------------

;Misc
tr = [min((*ds[0]).time), last.end_time]
for i=0, n_elements(ds)-1 do begin
  ;dist type name
  dname = keyword_set(dname) ? [dname,((*ds[i]).data_name)[0]]:((*ds[i]).data_name)[0]
  ;time range of data
  tr = [ tr[0] < min((*ds[i]).time), $
         tr[1] > max((*ds[i]).end_time) ]
endfor
dname = strjoin(dname,'/')

;Velocity range
if keyword_set(displacement) then begin
  vrange = sqrt( vrange^2 - displacement^2 > 0.)
endif 


slice_info = {probe: probe, $    ;probe
              dist: dname, $     ;type of distribution (string)
              coord: coord, $    ;coordinates used (string)
              rot: rotation, $   ;rotation used (string)
              units: units, $    ;units (string)
              twin: timewin, $   ;length of slice in sec
              type: type, $      ;integer specifying slice type (0,2,3)
              range: orange, $   ;data range before interpolation
              trange: tr, $      ;total time range of used distributions
              ndists: n_elements((*ds[0])), $ ;# distributions used
              vrange: vrange , $              ;instrument velocity range
              shift: keyword_set(subtract) ? vbulk:0, $ ;for nn plotting
              bulk: keyword_set(vbulk) ? vbulk:0, $     ;bulk velocity vector
              coord_m: keyword_set(ct) ? ct:-1, $ ;DSL to COORD matrix
              rot_m: rot, $                       ;COORD to ROTATION matrix
              orient_m: keyword_set(mt) ? mt:-1 $ ;COORd/ROTATION to slice plane coords matrix 
              }

return

end
