;Function to load requested mag data
;and save in tplot variable, 
;returns 1 if successful.
;
function thm_ui_slice2d_getmag, type, probe, trange, name, suffix

      compile_opt idl2, hidden

  thm_load_fgm, level=2, coord='dsl', probe=probe, datatype=type, $
                trange=trange, suffix=suffix

  names = tnames('*'+suffix)
  
  if names[0] eq '' then return, 0b
  
  store_data, names, newname=name

  return, 1b
  
end



;Function to load requested velocity data 
;and save in tplot variable, returns 1 if 
;successful.
;
function thm_ui_slice2d_getvel, type, probe, trange, vel_name, suffix, err_msg=err_msg

      compile_opt idl2, hidden

  ; Assume type contains 'p???' designation (e.g. 'psif','peem') 
  type0 = stregex(type,'p[es][ei][frbm]',/extract)
  inst = strmid(type0,1,1)
  dtype = strmid(type0,3,1)
  coord = stregex(type, 'mag',/bool,/fold_case) ? 'mag':'dsl'

  ; Load requested velocify into tplot 
  if dtype eq 'm' then begin
    thm_load_mom, probe=probe, datatype = type0+'_velocity_'+coord, suffix=suffix, trange=trange

  endif else begin
    if inst eq 'e' then begin
      thm_load_esa, probe=probe, datatype = type0+'_velocity_'+coord, suffix=suffix, trange=trange
    endif else if inst eq s then begin 
;      thm_load_sst, probe=probe, datatype = type0+'_velocity_'+coord, suffix=suffix, trange=trange
    endif
  endelse

  ; Check for multiple quantities
  tname = tnames('*'+suffix)
  if n_elements(tname) gt 1 then begin
    ;check if multiple quantities were from thm_load_mom
    vind = where( stregex(tname,'velocity',/bool) and ~stregex(tname,'pt[ei]m',/bool), nv)
    if nv eq 1 then begin
      tname = tname[vind]
    endif else begin
      err_msg = 'Unknown error. Multiple velocity variables found.'
      return, 0b
    endelse
  endif
  
  ; Check for no data.
  if tname eq '' then begin
    err_msg = 'No velocity data availabe for requested time range.'
    return, 0b
  endif
  
  store_data, tname[0], newname=vel_name ;rename variable
    
  return, 1b

end



; Checks for valid numbers 
; *Should be used if number is being pulled from a plain
;  text field instead of a spinner widget.
;
function thm_ui_slice2d_checknum, num

    compile_opt idl2, hidden

  if is_numeric(num) then begin
      on_ioerror, fail
    return, double(num)
    fail: return, !values.D_NAN
  endif else begin
    return,!values.D_NAN
  endelse

end



;Standard error message handling for thm_ui_slice2d_gen
;
pro thm_ui_slice2d_error, sb, title, err_msg

      compile_opt idl2, hidden

    ok = dialog_message(err_msg,/center,title=title)
    thm_ui_slice2d_message, err_msg, sb=sb
    
end



;Function to check whether distribution array needs
; to be re-loaded. 
;
function thm_ui_slice2d_check, tlb, state, previous

    compile_opt idl2, hidden

  ; Were any slices produced last time?
  ;
  if state.flags.forcereload then return, 0b

  ; Check probe
  ;
  id = widget_info(tlb, find_by_uname='probe')
  probeidx = widget_info(id, /list_select)
  if probeidx ne previous.probeidx then return, 0b
  
  
  ; Check distribution type
  ;
  id = widget_info(tlb, find_by_uname='dtype')
  didx = widget_info(id, /list_select)
  if ~array_equal(didx, previous.didx) then return, 0b


  ; Check B-field data option
  ;
  id = widget_info(tlb, find_by_uname='mag')
  if widget_info(id, /sens) then begin
    if widget_info(id, /combobox_gettext) ne previous.mag then return, 0b
  endif ;if desensitized mag data is irrelevant


  ; Check velocity data option
  ;
  id = widget_info(tlb, find_by_uname='vtype')
  if widget_info(id, /sens) then begin
    if strlowcase(widget_info(id, /combobox_gettext)) ne previous.vtype then return, 0b
  endif

  ; Check time range
  ;
  id = widget_info(tlb, find_by_uname='time')
  widget_control, id, get_value=time_obj
  t0 = time_obj->getstarttime()
  t1 = time_obj->getendtime()
  if t0 lt previous.trange[0] then return, 0b
  if t1 gt previous.trange[1] then return, 0b
  
  ;Check ESA background options
  ;
  id = widget_info(tlb, find_by_uname='esaremovebase')
  if widget_info(id, /sens) ne previous.esa_remove then return, 0b

  if widget_info(id, /sens) then begin
    id = widget_info(tlb, find_by_uname='esatype')
    bgnd_type = widget_info(id, /combobox_gettext)
    if bgnd_type ne previous.bgnd_type then return, 0b
    
    id = widget_info(tlb, find_by_uname='esanpoints')
    widget_control, id, get_value = bgnd_npoints
    if bgnd_npoints ne previous.bgnd_npoints then return, 0b
    
    id = widget_info(tlb, find_by_uname='esascale')
    widget_control, id, get_value = bgnd_scale
    if bgnd_scale ne previous.bgnd_scale then return, 0b
  endif
  
  ;Check SST mask option
  ;
  id = widget_info(tlb, find_by_uname='sstmask')
  if widget_info(id, /sens) ne previous.sst_mask then return, 0b
  
  if widget_info(id, /sens) then begin
    widget_control, id, get_value = mask_remove
    if mask_remove ne previous.mask_remove then return, 0b
  endif
  
  ;Check SST contamintion options
  ;
  id = widget_info(tlb, find_by_uname='sstremovebase')
  if widget_info(id, /sens) ne previous.sst_remove then return, 0b
  
  if widget_info(id, /sens) then begin
    id = widget_info(tlb, find_by_uname='sstmethod')
    method_sunpulse_clean = widget_info(id, /combobox_gettext)
    if method_sunpulse_clean ne previous.method_sunpulse_clean then return, 0b
    
    id = widget_info(tlb, find_by_uname='ssttolerance')
    widget_control, id, get_value = limit_sunpulse_clean
    if limit_sunpulse_clean ne previous.limit_sunpulse_clean then return, 0b
    
    id = widget_info(tlb, find_by_uname='sstfill')
    fillin_method = widget_info(id, /combobox_gettext)
    if fillin_method ne previous.fillin_method then return, 0b
  endif
  
  return, 1b

end



;Procedure called to generate slice
;
pro thm_ui_slice2d_gen, tlb, state

      compile_opt idl2, hidden

  ;initializations
  vel_auto = 0b
  vel_data = ''
  mag_data = ''
  mag_name = '2dslice_temp_mag'
  vel_name = '2dslice_temp_vel'
  temp_suffix =  '_2dslice_temp'
  erange=0
  fail=''
  more=''
  err_title='Error Generating Slice'

  ; skip re-loading distributions?
  skip = thm_ui_slice2d_check(tlb, state, state.previous)
  
  ; if no slices are produced this time then ensure the data
  ; is reloaded next time (double check against option changes)
  state.flags.forcereload = 1b

;t00=systime(/sec)
  ;Retrieve options from widgets (this may be a bit long...) 
  ;----------------------------------------------------

  ; Get rotation type
  id = widget_info(tlb, find_by_uname='rot')
  rotation = widget_info(id, /combobox_gettext)
      
  
  ; Get reolution (# points/dimension) for final plot
  id = widget_info(tlb, find_by_uname='resolution')
  if widget_info(id, /sens) then begin
    widget_control, id, get_value=resolution
    if ~finite(resolution) || resolution lt 10 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Invalid resolution, the plot resolution should be >= 10. Operation canceled.'
      return
    endif
  endif
  
  
;  ; Get slice width if applicable
;  if widget_info(state.width, /sens) then begin
;    widget_control, state.width, get_value=slice_width
;    if ~finite(slice_width) or slice_width le 0 or slice_width gt 100 then begin
;      thm_ui_slice2d_error, state.statusbar, err_title, $
;        'Invalid width, operation canceled.'
;      return
;    endif
;  endif  
  
  
  ; Type of interpolation
;  id = widget_info(tlb, find_by_uname='button2dnn')
;  if widget_info(id, /button_set) then type=1
  id = widget_info(tlb, find_by_uname='button2di')
  if widget_info(id, /button_set) then type=2
  id = widget_info(tlb, find_by_uname='button3di')
  if widget_info(id, /button_set) then type=3
  

  ; Ensure resolution is set
  if ~keyword_set(resolution) then begin
    resolution = keyword_set(type) ? (type eq 2 ? 51:150):500
  endif

  
  ; Get regrid option
  id = widget_info(tlb, find_by_uname='regridbase')
  if widget_info(id, /sens) then begin
    id = widget_info(tlb, find_by_uname='regridphi')
    widget_control, id, get_value=regridphi 
  
    id = widget_info(tlb, find_by_uname='regridtheta')
    widget_control, id, get_value=regridtheta 
    
    id = widget_info(tlb, find_by_uname='regride')
    widget_control, id, get_value=regride
    
    regrid = [thm_ui_slice2d_checknum(regridphi),   $
              thm_ui_slice2d_checknum(regridtheta), $
              thm_ui_slice2d_checknum(regride) ]

    if in_set(finite(regrid) and (regrid ge 8), 0) then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Invalid regrid values; all values must be >= 8. Operation canceled.'
      return
    endif
     
  endif else begin
  
    regrid=0
        
  endelse
  
  
  ; Get energy range limits if any
  if widget_info(state.erangebase, /sens) then begin
    id = widget_info(tlb, find_by_uname='emin')
    widget_control, id, get_value = emin
    id = widget_info(tlb, find_by_uname='emax')
    widget_control, id, get_value = emax
    
    if ~finite(emin) || ~finite(emax) || emin lt 0 || emax le 0 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Invalid energy range, operation canceled.'
      return
    endif
    if emin ge emax then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Maximum energy must be greater than minimum, operation canceled.'
      return
    endif
    
    erange = [emin, emax]
  endif


  ; Get theta/z range limits if any
  if widget_info(state.rangebase2d, /sens) then begin
    if widget_info(state.thetarangebase, /sens) then begin
      id = widget_info(tlb, find_by_uname='thetamin')
      widget_control, id, get_value = thetamin
      id = widget_info(tlb, find_by_uname='thetamax')
      widget_control, id, get_value = thetamax
      
      if ~finite(thetamin) || ~finite(thetamax) || thetamin lt -90 || thetamax gt 90 then begin
        thm_ui_slice2d_error, state.statusbar, err_title, $
          'Latitude must be between [-90,90] degrees, operation canceled.'
        return
      endif
      if thetamin ge thetamax then begin
        thm_ui_slice2d_error, state.statusbar, err_title, $
          'Maximum latitude angle must be greater than minimum, operation canceled.'
        return
      endif
      
      thetarange = [thetamin, thetamax]
    endif else begin
      id = widget_info(tlb, find_by_uname='zdirmin')
      widget_control, id, get_value = zdirmin
      id = widget_info(tlb, find_by_uname='zdirmax')
      widget_control, id, get_value = zdirmax
      
      if ~finite(zdirmin) || ~finite(zdirmax) then begin
        thm_ui_slice2d_error, state.statusbar, err_title, $
          'Invalid Z-axis restrictions, operation canceled.'
        return
      endif
      if zdirmin ge zdirmax then begin
        thm_ui_slice2d_error, state.statusbar, err_title, $
          'Maximum latitude angle must be greater than minimum, operation canceled.'
        return
      endif
      
      zdirrange = [zdirmin, zdirmax]
    endelse
  endif

  
  ; Subtract bulk velocity?
  id = widget_info(tlb, find_by_uname='subtract')
  subtract = widget_info(id, /button_set)
    
  ; Remove below one count?
  id = widget_info(tlb, find_by_uname='onecount')
  onecount = widget_info(id, /button_set) 
  
  ; Smoothing
  id = widget_info(tlb, find_by_uname='smooth_width')
  if widget_info(id, /sens) then begin
    widget_control, id, get_value=smooth
    if ~finite(smooth) || (smooth le 2) || (smooth gt resolution/2.) then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Invalid smoothing window width.'
      return
    endif 
  endif else smooth = 0

  
  
  ; Get slice's normal vector
  slice_norm = [-1d,-1d,-1d]
  unames = ['orx','ory','orz']
  for i=0, n_elements(slice_norm)-1 do begin
    id = widget_info(tlb, find_by_uname=unames[i])
    widget_control, id, get_value = temp
    slice_norm[i] = thm_ui_slice2d_checknum(temp)
  endfor
  if in_set(0,finite(slice_norm)) then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Invalid normal vector, operation canceled.'
    return
  endif
  if norm(slice_norm) lt 5e-6 then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Invalid normal vector, length must be > 0.'
    return
  endif
  
  ; Get displacement from origin
  id = widget_info(tlb, find_by_uname='displace')
  widget_control, id, get_value=displacement
  if ~finite(displacement) then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Invalid displacement, operation canceled.'
    return
  endif 
  
  
  ; Get Time Options
  ;
 
  ; get length of time window
  id = widget_info(tlb, find_by_uname='timewin')
  widget_control, id, get_value = timewin
  if ~finite(timewin) || timewin le 0 then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Invalid time window, please re-enter.'
    return
  endif
  
  ; get time incriment
  id = widget_info(tlb, find_by_uname='timeinc')
  widget_control, id, get_value = timeinc
  if ~finite(timeinc) || timeinc le 0 then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Invalid time increment, please re-enter.'
    return
  endif
  
  ;get center option
  id = widget_info(tlb, find_by_uname='center')
  center_time = widget_info(id, /button_set) 
  
  ; get selected time range and set slice windows
  id = widget_info(tlb, find_by_uname='time')
  widget_control, id, get_value=validtime, func_get_value='thm_ui_time_widget_is_valid'
  if ~validtime then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Invalid time range, please re-enter.'
    return
  endif
  widget_control, id, get_value=time_obj, func_get_value='thm_ui_time_widget_get_value'
  t0 = time_obj->getstarttime()
  t1 = time_obj->getendtime()
  trange = [t0,t1]
  trange_p = trange + 120*[-1,1] ; pad time range by 2 minutes on each end to ensure 
                                 ; interpolated velocity data does not contain NaNs
  if timewin gt 90 then trange_p += (timewin-90)*[-1,1] ; add extra padding for
                                                        ; long time windows

  ; create start times for slices
  for t = t0, t1, timeinc do begin
    if t eq t0 then times = [t] $
      else times = [times,t]
  endfor
  if center_time then timeidx = where(times le t1, c) else $
    timeidx = where(times lt t1, c)
  if c ne 0 then times = times[timeidx]
  ntimes = n_elements(times)

  
  ; Determing probe
  id = widget_info(tlb, find_by_uname='probe')
  probeidx = widget_info(id, /list_select)
  if probeidx ge 0 then begin 
    widget_control, id, get_uvalue=val
    probe = val[probeidx]
  endif else begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'No probe selected, operation canceled.'
    return
  endelse
  
  
  ; Determine distribution type
  id = widget_info(tlb, find_by_uname='dtype')
  didx = widget_info(id, /list_select)
  if didx[0] ge 0 then begin 
    widget_control, id, get_uvalue=val
    dtype = val[didx]
    if keyword_set(type) and n_elements(didx) gt 1 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Multiple distributions are only supported for the '+ $
        'geometric method.'
      return
    endif 
    if n_elements(didx) gt 4 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'No more than 4 distribution types can be combined.'
      return
    endif
  endif else begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'No particle distribution selected, operation canceled.'
    return
  endelse

  
  ; Determine units type
  id = widget_info(tlb, find_by_uname='utype')
  uidx = widget_info(id, /list_select)
  if uidx ge 0 then begin 
    widget_control, id, get_uvalue=val
    units = val[uidx]
  endif else begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'No units selected, operation canceled.'
    return
  endelse


  ; Determine coordinates
  id = widget_info(tlb, find_by_uname='ctype')
  cidx = widget_info(id, /list_select)
  if cidx ge 0 then begin 
    widget_control, id, get_uvalue=val
    coord = val[cidx]
  endif 


  ; Use new sst calibrations?
  id = widget_info(tlb, find_by_uname='sstcal')
  sst_cal = widget_info(id, /button_set)

  ; SST sunpulse remove?
  id = widget_info(tlb, find_by_uname='sstremovebase')
  sst_remove = widget_info(id, /sens)

  ; SST mask proportion
  id = widget_info(tlb, find_by_uname='sstmask')
  sst_mask = widget_info(id, /sens)
  if sst_mask then begin
    widget_control, id, get_value = mask_remove
    if ~finite(mask_remove) or mask_remove gt 1 or mask_remove lt 0 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Invalid SST mask proportion (must be in [0,1]).'
      return
    endif
  endif
  
  ; SST Sun removal method
  id = widget_info(tlb, find_by_uname='sstmethod')
  if widget_info(id, /sens) then begin
    method_sunpulse_clean = widget_info(id, /combobox_gettext)
  endif

  ; SST contamination tolerance
  id = widget_info(tlb, find_by_uname='ssttolerance')
  if widget_info(id, /sens) then begin
    widget_control, id, get_value = limit_sunpulse_clean
    if ~finite(limit_sunpulse_clean) || limit_sunpulse_clean le 0 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Invalid SST contamination removal tolerance.'
      return
    endif
  endif

  ; SST fill method
  id = widget_info(tlb, find_by_uname='sstfill')
  if widget_info(id, /sens) then begin
    fillin_method = widget_info(id, /combobox_gettext)
  endif


  ; ESA background removal
  id = widget_info(tlb, find_by_uname='esaremovebase')
  esa_remove = widget_info(id, /sens)
  if esa_remove then bgnd_remove = esa_remove
  
  ; ESA removal type
  id = widget_info(tlb, find_by_uname='esatype')
  if widget_info(id, /sens) then begin
    bgnd_type = widget_info(id, /combobox_gettext)
  endif
  
  ; ESA # points (bngd removal)
  id = widget_info(tlb, find_by_uname='esanpoints')
  if widget_info(id, /sens) then begin
    widget_control, id, get_value = bgnd_npoints
    if ~finite(bgnd_npoints) || bgnd_npoints lt 1 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Invalid number of pounts for ESA background removal.'
      return
    endif
  endif
  
  ; ESA background scale factor
  id = widget_info(tlb, find_by_uname='esascale')
  if widget_info(id, /sens) then begin
    widget_control, id, get_value = bgnd_scale
    if ~finite(bgnd_scale) || bgnd_scale le 0 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Invalid scale factor for ESA background removal.'
      return
    endif
  endif


  ; Get array of particle distributions
  ; ------------------------------------
  
  ; Skip re-loading distributions if data type, probe, 
  ; mag data selection, vel data, contamination removal 
  ; options remain unchanged and time range is within previous.
  if ~skip then begin 

    thm_ui_slice2d_message, 'Loading Distributions...', sb=state.statusbar


    ; Get B-field data if needed
    id = widget_info(tlb, find_by_uname='mag')
    magsens = widget_info(id, /sens) 
    if magsens then begin
      mag_data = mag_name
      mag = widget_info(id, /combobox_gettext)
      if ~thm_ui_slice2d_getmag(mag, probe, trange_p, mag_data, temp_suffix) then begin
        thm_ui_slice2d_error, state.statusbar, err_title, $
          'No '+mag+' data for requested time interval.'
        return
      endif
    endif
  
  
    ; Get requested velocity data for distribution if required
    ;
    if widget_info(state.velbase, /sens) then begin
      id = widget_info(tlb, find_by_uname='vtype')
      vtype = strlowcase(widget_info(id, /combobox_gettext))
      if vtype eq 'auto' then begin
        vel_auto=1b
      endif else begin
        vel_data = vel_name ;set this for later
        if ~thm_ui_slice2d_getvel(vtype, probe, trange_p, $
                                  vel_name, temp_suffix, err_msg=err_msg) then begin
          if ~keyword_set(err_msg) then err_msg = 'Unknown error loading velocity data.'
          thm_ui_slice2d_error, state.statusbar, err_title, err_msg
          return
        endif
      endelse
    endif else begin
      vel_auto=1b
    endelse
  
    
    ; Create arrays of particle distributions
    ;
    for i=0, n_elements(state.distribution)-1 do begin
      ptr_free, state.distribution[i]
      
      if i lt n_elements(dtype) then begin
        distribution = thm_part_dist_array(type=dtype[i], probe=probe, trange=trange, $
                          mag_data=mag_data, vel_data=vel_data, vel_auto=vel_auto, $
                          suffix=temp_suffix, err_msg=err_msg, $
                          ;esa background
                          bgnd_remove=bgnd_remove, bgnd_type=bgnd_type, $
                          bgnd_npoints=bgnd_npoints, bgnd_scale=bgnd_scale, $
                          ;use new sst cal
                          sst_cal=sst_cal, $
                          ;sst contamination
                          mask_remove=mask_remove, fillin_method=fillin_method, $
                          method_sunpulse_clean=method_sunpulse_clean, $
                          limit_sunpulse_clean=limit_sunpulse_clean)
        
        
        ; Check for errors from thm_part_dist_array
        if size(distribution,/type) ne 8 then begin
          if ~keyword_set(err_msg) then err_msg='Unknown error loading: '+dtype[i]+' data.'
          thm_ui_slice2d_error, state.statusbar, err_title, err_msg
          continue
        endif else if keyword_set(err_msg) then begin
          ; issue error message, but continue if returned distributions are ok
          thm_ui_slice2d_error, state.statusbar, err_title, err_msg
        endif
        
        ; Save distribution
        state.distribution[i] = ptr_new(distribution)
      endif
    endfor
  
    if total(ptr_valid(state.distribution)) lt 1 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'No data could be loaded, slice generation canceled.'
      return
    endif
    
    ; Save settings
    previous = state.previous
    
    str_element, previous, 'didx', didx, /add
    previous.probeidx = probeidx
    previous.mag = size(mag,/type) ? mag:''
    previous.vtype = size(vtype,/type) ? vtype:''
    previous.trange = trange

    previous.esa_remove = esa_remove
    previous.bgnd_remove = size(bgnd_remove,/type) ? bgnd_remove:-1
    previous.bgnd_npoints = size(bgnd_npoints,/type) ? bgnd_npoints:-1d
    previous.bgnd_scale = size(bgnd_scale,/type) ? bgnd_scale:-1d
    previous.bgnd_type = size(bgnd_type,/type) ? bgnd_type:''
    
    previous.sst_mask = sst_mask
    previous.sst_remove = sst_remove
    previous.mask_remove = size(mask_remove,/type) ? mask_remove:-1d
    previous.method_sunpulse_clean = size(method_sunpulse_clean,/type) ? method_sunpulse_clean:''
    previous.fillin_method = size(fillin_method,/type) ? fillin_method:''
    previous.limit_sunpulse_clean = size(limit_sunpulse_clean,/type) ? limit_sunpulse_clean:-1d
    
    str_element, state, 'previous', previous, /add
    
  endif else begin ; Use old distribution if required options unchanged
  
    thm_ui_slice2d_message, 'Main options unchanged - loading saved distributions...', sb=state.statusbar
;    distribution = *state.distribution
    
  endelse  ;End skip block
  


  ; Generate data for plotting
  ;---------------------------
  
  ; Initialize arrays for slice data if using new method
  part_slice = fltarr(ntimes,resolution,resolution, /nozero)
  xgrid = fltarr(ntimes,resolution, /nozero)
  ygrid = fltarr(ntimes,resolution, /nozero)
  
  thm_ui_slice2d_message, 'Creating velocity slices...', sb=state.statusbar
  
  ; Populate input parameters to thm_part_slice2d
  for i=0, n_elements(state.distribution)-1 do begin
    if ptr_valid(state.distribution[i]) then begin
      x = [keyword_set(dist1),keyword_set(dist2), $
           keyword_set(dist3),keyword_set(dist4)]
      case total(x) of 
        0: dist1 = *state.distribution[i]
        1: dist2 = *state.distribution[i]
        2: dist3 = *state.distribution[i]
        3: dist4 = *state.distribution[i]
        else:
      endcase
    endif
  endfor
  
  ; Populate data arrays
  bad = bytarr(ntimes)
  for i=0, ntimes-1 do begin
;t11 = systime(/sec)
    thm_part_slice2d, dist1, dist2, dist3, dist4, $
                      slice_time=times[i], timewin=timewin, $
                      center_time=center_time, $ 
                      slice_norm=slice_norm, $
                      displacement = displacement, $
                      rotation=rotation, resolution=resolution, $
                      type=type, $
                      coord=coord, $
                      erange=erange, $
                      zdirrange=zdirrange, thetarange=thetarange, $
;                      slice_width=slice_width, $
                      subtract=subtract, $
                      onecount=onecount, $
                      units=units, $
                      smooth=smooth, $
                      regrid=regrid, $
                      part_slice=part_slice_tmp, $
                      xgrid=xgrid_tmp, $
                      ygrid=ygrid_tmp, $
                      slice_info=slice_info_tmp, $
                      fail=fail
                      
    
    ; Check for errors from thm_part_slice2d and skip those slices
    if fail then begin
      bad[i] = 1b
      fail = 'Cannot create slice at '+time_string(times[i])+' - '+fail
      thm_ui_slice2d_message, fail, sb=state.statusbar
      continue
    endif
    
    ; Add slice data to arrays (cannot add directly in thm_part_slice2d call)
    part_slice[i,*,*] = temporary(part_slice_tmp)
    xgrid[i,*] = temporary(xgrid_tmp)
    ygrid[i,*] = temporary(ygrid_tmp)

    ; Store slice info
    slice_info = ~keyword_set(slice_info) ? temporary(slice_info_tmp):[slice_info,temporary(slice_info_tmp)]

  endfor

  
  ; Plot
  ;---------

  ; put plotting elements into structure that can be easily referenced
  good = where(bad eq 0b, ngood, ncomplement=nbad)
  if ngood eq 0 then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'No valid slices could be created, see console output.'
    return
  endif
  
  slice_data = {part_slice:temporary(part_slice[good,*,*]), $
                xgrid:temporary(xgrid[good,*,*]), $
                ygrid:temporary(ygrid[good,*,*]), $
                slice_info:slice_info}
  ptr_free, state.slices
  state.slices = ptr_new(slice_data)

  ; if we made it to here, no need to force reload data next time
  state.flags.forcereload = 0b

  ; prep slider bar (start at first slice by default)
  state.slider->setProperty, range=[0,ngood-1], ok=1b
  state.slider->update
  state.slider->getProperty, value=current

  ; prep info
  ptr_free, state.times
  state.times = ptr_new(times[good])
  state.last = 'th'+probe+'_'+strjoin(dtype,'-')+'_'+rotation
  
  ; plot
  thm_ui_slice2d_plot, state, current

  ; update title
  s = ngood ne 1 ? 's':''
  widget_control, state.tlb, tlb_set_title = state.tlb_title + '  ('+strtrim(ngood,2)+' plot'+s+')' 

  if nbad ne 0 then more += ' - Some slices could not be produced, see console output' 
  thm_ui_slice2d_message,'Finished'+more, sb=state.statusbar

;  heap_gc ; clean up memory

end



;Plots current slice with given options
;
pro thm_ui_slice2d_plot, state, current, _extra=_extra

    compile_opt idl2, hidden

  tlb = state.tlb
  err_title = 'Error Creating Plot'
  
  if ~ptr_valid(state.slices) then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'No slices currently loaded, operation canceled.'
    return
  endif

  ; Update fixed-range spinners
  thm_ui_slice2d_rangeupdate, tlb, state

  ; Get Z-range limits & options
  if widget_info(state.zminmaxbase, /sens) then begin
    id = widget_info(tlb, find_by_uname='zmin')
    widget_control, id, get_value = zmin
    id = widget_info(tlb, find_by_uname='zmax')
    widget_control, id, get_value = zmax
    
    if ~finite(zmin) || ~finite(zmax) || zmin lt 0 || zmax le 0 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Invalid z-axis limits, operation canceled.'
      return
    endif
    if zmin ge zmax then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Maximum z-axis limit must be greater than the minimum, operation canceled.'
      return
    endif
    range = [zmin, zmax]
  endif

  ; Get XY-range limits & options
  if widget_info(state.xyminmaxbase, /sens) then begin
    id = widget_info(tlb, find_by_uname='xymin')
    widget_control, id, get_value = xymin
    id = widget_info(tlb, find_by_uname='xymax')
    widget_control, id, get_value = xymax
    
    if ~finite(xymin) || ~finite(xymax) then begin
      thm_ui_slice2d_error, state.statusbar, err_tittle, $
        'Invalid x-y axis limits, operations canceled.'
      return
    endif
    if xymin ge xymax then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Maximum X,Y axis limit must be greater than minimum, operation canceled.'
      return
    endif
    xrange = [xymin, xymax]
    yrange = [xymin, xymax]
  endif

  ; Get contour lines options
  id = widget_info(tlb, find_by_uname='nolines')
  if widget_info(id, /sensitive) then begin
    widget_control, id, get_value=olines
    if ~finite(olines) || olines lt 0 then begin
      thm_ui_slice2d_error, state.statusbar, err_title, $
        'Invalid number of contour levels, operation canceled.'
      return
    endif 
  endif else begin
    olines = 0
  endelse 

  ; Get contour colors options
  id = widget_info(tlb, find_by_uname='nlevels')
  widget_control, id, get_value=nlines
  if ~finite(nlines) || nlines le 0 then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Invalid number of color levels, operation canceled.'
    return
  endif 

  ; Get character size
  id = widget_info(tlb, find_by_uname='charsize')
  widget_control, id, get_value=charsize
  charsize /= 100
  if ~finite(charsize) || charsize le 0 then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Invalid character size, operation canceled.'
    return
  endif

  ; Get # of ticks
  id = widget_info(tlb, find_by_uname='nxticks')
  if widget_info(id, /sens) then widget_control, id, get_value = xticks
  id = widget_info(tlb, find_by_uname='nyticks')
  if widget_info(id, /sens) then widget_control, id, get_value = yticks
  id = widget_info(tlb, find_by_uname='nzticks')
  if widget_info(id, /sens) then widget_control, id, get_value = zticks
  if size(xticks,/type) then begin
    if ~finite(xticks) || (xticks lt 0) || (xticks gt 60) then tick_err = 1b
  endif
  if size(yticks,/type) then begin
    if ~finite(yticks) || (yticks lt 0) || (yticks gt 60) then tick_err = 1b
  endif
  if size(zticks,/type) then begin
    if ~finite(zticks) || (zticks lt 0) || (zticks gt 60) then tick_err = 1b
  endif
  if keyword_set(tick_err) then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Number of ticks must be between 0 and 60, operation canceled.'
    return
  endif

  ; Get annotation precision
  id = widget_info(tlb, find_by_uname='precision')
  precision = widget_info(id, /combobox_gettext)
  widget_control, id, get_value=precisions
  precisionindex = where(precision eq precisions, c)
  if c eq 0 then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Cannot identify annotation precision, operation canceled.'
    return
  endif

  ;Get annotation style
  for i=0, 2 do begin
    id = widget_info(tlb, find_by_uname='anno'+strtrim(i,2))
    if widget_info(id, /button_set) then style=i
  endfor
  if size(style,/type) eq 0 then begin
    thm_ui_slice2d_error, state.statusbar, err_title, $
      'Cannot determine annotation style, operation canceled.'
    return
  endif

  ; Get log option
  id = widget_info(tlb, find_by_uname='logplot')
  logplot = widget_info(id, /button_set)

  ; Get sun vector option
  id = widget_info(tlb, find_by_uname='sundir')
  sundir = widget_info(id, /button_set)

  ; Get bulk velocity option
  id = widget_info(tlb, find_by_uname='plotbulk')
  plotbulk = widget_info(id, /button_set)

  ; Get min/max energy circle option
  id = widget_info(tlb, find_by_uname='ecirc')
  ecircle = widget_info(id, /button_set)

  ; Get plot axes option
  id = widget_info(tlb, find_by_uname='axes')
  plotaxes = widget_info(id, /button_set)

  ; Get label contours option
  id = widgeT_info(tlb, find_by_uname='labelcontours')
  clabels = widget_info(id, /button_set)

  
  ; Plot
  ;
  plot_part_slice2d, reform((*state.slices).part_slice[current,*,*]), $
                     reform((*state.slices).xgrid[current,*]), $
                     reform((*state.slices).ygrid[current,*]), $
                     (*state.slices).slice_info[current], $
                     olines=olines, $
                     nlines=nlines, $
                     range=range, $
                     xrange=xrange, $
                     yrange=yrange, $
                     logplot=logplot, $
                     sundir=sundir, $
                     plotbulk=plotbulk, $
                     ecircle=ecircle, $
                     plotaxes=plotaxes, $
                     clabels = clabels, $
                     charsize=charsize, $
                     aprecision=precisionindex, $
                     astyle=style, $
                     xticks=xticks, $
                     yticks=yticks, $
                     zticks=zticks, $
                     _extra=_extra
                     

end



; Export slice(s) to .png
;
pro thm_ui_slice2d_export, state, uval, all=all

    compile_opt idl2, hidden


  ; check that current data is valid
  if ~ptr_valid(state.slices) then begin
    thm_ui_slice2d_error, state.statusbar, 'Export Error:', $
      'There are no plots to export, operation canceled.'
    return
  endif

  ; check that plotting window is open if exporting one
  if ~keyword_set(all) and !d.window lt 0 then begin
    thm_ui_slice2d_error, state.statusbar, 'Export Error:', $
      'There is no plot currently open, operation canceled.'
    return
  endif

  ; pull default name from last plot
  last = state.last
  filename = last eq '' ? 'new_slice':last
  filename = '(slice_time)_'+filename
  etypes = ['png','eps']
  
  ; get file type
  etype = strlowcase( stregex(uval, '('+strjoin(etypes,'|')+')$', /extract, /fold_case) )
  if etype eq '' then etype = etypes[0]

  ; get file path
;  filepath = dialog_pickfile(title='Export Slice', filter=('*.'+etype), $
;                         file=filename, /write, dialog_parent=state.gui_id, $
;                         /overwrite_prompt)
  filepath = thm_ui_dialog_pickfile_save_wrapper(title='Export Slice', filter=('*.'+etype), $
                         file=filename, /write, dialog_parent=state.tlb, $
                         /overwrite_prompt)

  ; continue if file path ok
  if is_string(filepath) then begin

    ;strip extension - will be added regardless
    ext_idx = stregex(filepath, '\.'+etypes[0]+' *$', /fold_case)
    if ext_idx gt 0 then filepath = strmid(filepath,0,ext_idx)
    ext_idx = stregex(filepath, '\.'+etypes[1]+' *$', /fold_case)
    if ext_idx gt 0 then filepath = strmid(filepath,0,ext_idx)
    
    ;insert slice time if flag still exists in filepath
    marker_idx = stregex(filepath, '\(slice_time\)', length=mlength, /fold_case)
    if marker_idx ge 0 then begin
      pathname = strmid(filepath,0,marker_idx)
      filename = strmid(filepath,marker_idx+mlength)
      dynamic = 1b
    endif else begin
      file = filepath
      dynamic = 0b
    endelse

    ;create image(s)
    if ~keyword_set(all) then begin
      if dynamic then begin
        state.slider->getProperty, value=current
        file = pathname + time_string((*state.times)[current],format=2) + filename
      endif
      
      ; check for overwrite
      if file_test(file+'.'+etype) then begin
        overwrite = thm_ui_prompt_widget(state.gui_id, (obj_new()), state.historywin, $
                       promptText='Overwrite file: '+file+' ?', /yes, /no)
        if overwrite eq 'no' then begin
          thm_ui_slice2d_message,'Export Canceled', sb=state.statusbar
          return
        endif
      endif

      
      ; export to .png
      if etype eq etypes[0] then begin
        makepng, file, mkdir=1b, no_expose=1b
      endif
      
      ; export to .eps
      if etype eq etypes[1] then begin
        state.slider->getProperty, value=current
        popen, file, encapsulated = 1
        thm_ui_slice2d_plot, state, current
        pclose
      endif
      
    endif else begin
    
      for j=0, n_elements(*state.times)-1 do begin
        if dynamic then begin
          file = pathname + time_string((*state.times)[j],format=2) + filename
          suffix = ''
        endif else begin 
          suffix = '_'+strtrim(j,2)
        endelse
        
         ; check for overwrite
        if file_test(file+suffix+'.'+etype) then begin
          if keyword_set(allno) then begin
            thm_ui_slice2d_message,'Export - Skipping: '+file+suffix+'.'+etype, sb=state.statusbar
            continue
          endif
          if ~keyword_set(allyes) then begin
            overwrite = thm_ui_prompt_widget(state.gui_id, (obj_new()), state.historywin, $
                           promptText='Overwrite file: '+file+' ?', /yes, /no, /allyes, /allno, /cancel)
            if overwrite eq 'cancel' then begin
              thm_ui_slice2d_message,'Export Canceled', sb=state.statusbar
              return
            endif
            allyes = overwrite eq 'yestoall'
            allno = overwrite eq 'notoall'
            if overwrite eq 'no' or allno then begin
              thm_ui_slice2d_message,'Export - Skipping: '+file+suffix+'.'+etype, sb=state.statusbar
              continue
            endif
          endif
        endif 
        
        ; export all to .png
        if etype eq etypes[0] then begin
          thm_ui_slice2d_plot, state, j
          makepng, file+suffix, mkdir=1b, no_expose=1b
        endif
        
        ; export all to .eps
        if etype eq etypes[1] then begin
          popen, file+suffix, encapsulated = 1
          thm_ui_slice2d_plot, state, j
          pclose
        endif
      endfor
      
    endelse
    
    thm_ui_slice2d_message,'Export Successful', sb=state.statusbar
  endif else begin
    thm_ui_slice2d_message,'Export Canceled', sb=state.statusbar
  endelse

end



; Legend for rotation option
;
pro thm_ui_slice2d_rotlegend, gui_ID, top_ID

    compile_opt idl2, hidden


  m = gui_id ne top_id ; top base must be mobile if group leader is as well
  
  tlb2 = widget_base(title = 'Ion/Electron Velocity Slices - Rotations Legend', $
                     /base_align_center, /col, modal=m, float=~m, group_leader=gui_id)
  
  mainBase = widget_base(tlb2, /col, xpad=4, ypad=4, space=2, tab_mode=1)
  
  cr = thm_ui_newline()
  text = ['BV: The x axis is parallel to B field; the bulk velocity defines the x-y plane.', $
   'BE: The x axis is parallel to B field; the B x V(bulk) vector defines the x-y plane.', $
   'xyz: The x axis is V_x and the y axis is V_y.', $
   'xvel: The x axis is V_x; the x-y plane is defined by the bulk velocity.', $ 
   'perp: The x axis is the bulk velocity projected onto the plane normal to the B field; y is B x V(bulk).', $
   'perp_xyz: Geometric xy coordinates projected onto the plane normal to the B field.', $
   ' ', $
   'Rotations such as "zx" and "perp_yz" can be replicated by selecting the "xyz"/"perp_xyz" option and ', $
   ' changing the orientation vector (the slice plane normal) to correspond with the desired rotation.', $
   '    **e.g. <1,0,0> will replicate a "yz" rotation']


  ;cross platform size control
  mx = max(strlen(text), mi)
  dummy = widget_label(mainbase, value=text[mi])
  geo = widget_info(dummy,/geo)
  widget_control, dummy, /destroy
  
  rotlegend = widget_label(mainbase, value=strjoin(text,cr), $ 
                        xsize=geo.scr_xsize, $
                        ysize=geo.scr_xsize/((1+sqrt(5))) )
  
  centertlb, tlb2
  
  widget_control, tlb2, /realize 

end



; Update specifies range spinners
;
pro thm_ui_slice2d_rangeupdate, tlb, state

    compile_opt idl2, hidden

  factor = [0.999,1.001] ;same factor used in plot_part_slice2d
  state.slider->getProperty, value=current

  ; update zrange if auto-range is on
  if ~widget_info(state.zminmaxbase, /sens) then begin
    zrange = minmax((*state.slices).slice_info.range,/pos)
    id = widget_info(tlb, find_by_uname='zmin')
    widget_control, id, set_value = zrange[0]*factor[0]
    id = widget_info(tlb, find_by_uname='zmax')
    widget_control, id, set_value = zrange[1]*factor[1]
  endif
  
  ; update x/y range if auto-range is on
  if ~widget_info(state.xyminmaxbase, /sens) then begin
    xrange = minmax((*state.slices).xgrid[current,*])
    yrange = minmax((*state.slices).xgrid[current,*])
    xyrange = minmax([xrange,yrange])
    id = widget_info(tlb, find_by_uname='xymin')
    widget_control, id, set_value = xyrange[0]
    id = widget_info(tlb, find_by_uname='xymax')
    widget_control, id, set_value = xyrange[1]
  endif 

end


; Properly sensitize options and set defaults 
; according to the slice type. 
; (see thm_part_slice2d)
; NB: initpanel is a workaround for problems using set_value to the rotation combobox on X11 before widget realised.
;

pro thm_ui_slice2d_sens, state, id, select, initpanel=initpanel

    compile_opt idl2, hidden

; determine new/old
uname = widget_info(id, /uname)
geo = (uname eq 'buttongeo')
near = (uname eq 'button2dnn')
three = (uname eq 'button3di')

; maintain logic regardless of how/when
; events are generated (cross platform)
new = near || three || geo
w = near && select
y = geo && select
s = ~( new xor select )

; apply to widgets
widget_control, state.rangebase2d, sens = ~s
;widget_control, state.orientationbase, sens = s
widget_control, state.displacementbase, sens = s
;widget_control, state.width, sens = w

;NEED TO FIX SUBRACT BULK OPTION FOR GEOMETRIC
id = widget_info(state.tlb, find_by_uname='subtract')
widget_control, id, sens = ~geo

;smoothing
id = widget_info(state.tlb, find_by_uname='smooth')
widget_control, id, set_button = ~s
id = widget_info(state.tlb, find_by_uname='smoothbase')
widget_control, id, sens = ~s

;contour lines
if ~state.flags.olinebutton then begin
  id = widget_info(state.tlb, find_by_uname='olines')
  widget_control, id, set_button = ~s 
  id = widget_info(state.tlb, find_by_uname='nolines')
  widget_control, id, sens = ~s
endif
if ~state.flags.olineset then begin
  id = widget_info(state.tlb, find_by_uname='nolines')
  widget_control, id, set_value = ~s ? '20':'8'
endif

;resolution
if ~state.flags.resset then begin
  id = widget_info(state.tlb, find_by_uname='resolution')
  widget_control, id, set_value = s ? (y ? '500':'150'):'51'
endif


;regridding
id = widget_info(state.tlb, find_by_uname='regrid')
widget_control, id, set_button = w
id = widget_info(state.tlb, find_by_uname='regridbase')
widget_control, id, sens = w  
widget_control, state.regridbase, sens = ~geo


;; change rotation options
;id = widget_info(state.tlb, find_by_uname='rot')
;current = widget_info(id, /combobox_gettext)
;if new then begin
;  if ~keyword_set(initpanel) then widget_control, id, set_value=state.rotshort
;  xyzloc = stregex(current,'([^xyz]|^)([xyz]{2}$)',/subexpr)
;  if xyzloc[2] ge 0 then current = strmid(current,0,xyzloc[2]) + 'xyz'
;  cidx = where(state.rotshort eq current, cn)
;  if cn ne 0 then widget_control, id, set_combobox_select=cidx[0]
;endif else begin
;  widget_control, id, set_value=state.rotlong
;  xyzloc = stregex(current,'([^xyz]|^)([xyz]{3}$)',/subexpr)
;  if xyzloc[2] ge 0 then current = strmid(current,0,xyzloc[2]) + 'xy'
;  cidx = where(state.rotlong eq current, cn)
;  if cn ne 0 then widget_control, id, set_combobox_select=cidx[0]
;endelse


end



; Properly sensitize bulk velocity option
; and mag data option
;
pro thm_ui_slice2d_velsens, state

    compile_opt idl2, hidden
    
 
  id = widget_info(state.tlb, find_by_uname='rot')
  current = widget_info(id, /combobox_gettext)
  id = widget_info(state.tlb, find_by_uname='subtract')
  sub = widget_info(id, /button_set)
  
  ;option only valid for  BV, BE, xvel, and perp
  if in_set(state.rotvel, current) then begin
    sens = 1b
  endif else begin
    sens = 0b
  endelse
  
  ;always sensitize if set explicitly
  if sub then sens = 1b

  ;desensitize mag option when not relevant
  if in_set(state.rotmag, current) then begin
    msens = 1b
  endif else begin
    msens = 0b
  endelse

  widget_control, state.velbase, sens=sens
  widget_control, state.magbase, sens=msens
  
end



; Properly sensitize ticks options
;
pro thm_ui_slice2d_ticksens, event, uval

    compile_opt idl2, hidden

  ; user name of corresponding spinner should be "n" + UVAL
  id = widget_info(event.top, find_by_uname='n'+strlowcase(uval))
  widget_control, id, sens=event.select

end



; Properly sensitize contamination removal options
;
pro thm_ui_slice2d_contsens, event

    compile_opt idl2, hidden

  ; get button status
  id = widget_info(event.top, find_by_uname='esaremove')
  esa = widget_info(id, /button_set)
  
  id = widget_info(event.top, find_by_uname='sstmaskremove')
  sstm = widget_info(id, /button_set)
  
  id = widget_info(event.top, find_by_uname='sstremove')
  sst = widget_info(id, /button_set)

  ; get widget bases
  esabase = widget_info(event.top, find_by_uname='esaremovebase')
  sstmaskbase = widget_info(event.top, find_by_uname='sstmask')
  sstbase = widget_info(event.top, find_by_uname='sstremovebase')
  
  ; sensitize widgets
  widget_control, esabase, sens = esa
  widget_control, sstmaskbase, sens = sstm
  widget_control, sstbase, sens = sst

end



; Event handler
;
pro thm_ui_slice2d_event, event

    compile_opt idl2, hidden
  
  widget_control, event.top, get_uval=state, /no_copy
  
;error catch block
  catch, on_err
  if on_err ne 0 then begin
    catch, /cancel
    help, /last_message, output=msg
    print, msg
    
    if !d.name eq 'PS' then pclose ;ensure PS device closed
    
    store_data, tnames('*2dslice_temp*'), /delete ; delete any temp variables

    if is_struct(state) then begin

      ok=error_message('An unknown error occured and the current operation had to be canceled, '+ $
                       'see console for details.', /noname,/center,title='Error in 2D Slice Options')

      if widget_valid(state.gui_ID) and obj_valid(state.historywin) then begin
        for j=0, n_elements(msg)-1 do state.historywin->update,msg[j]
      endif

      ; reset these structures if state valid
      str_element, state, 'previous', /add, $
             {probeidx:-1,didx:-1,mag:'',vtype:'',trange:[-1d,-1d], $
              bgnd_remove:-1, bgnd_type:'', bgnd_npoints:-1d, bgnd_scale:-1d, $
              method_sunpulse_clean:'', limit_sunpulse_clean:-1d, $
              fillin_method:'', mask_remove:-1d, $
              esa_remove:-1, sst_remove:-1, sst_mask:-1}
      str_element, state, 'flags', /add, $
             {forcereload:1b,olineset:0b,olinebutton:0b,resset:0b}

      widget_control, event.top, set_uval=state, /no_copy

    endif else begin
      ok=error_message('An unknown error occured and the window must be restarted, see console for details.', $
                     /noname,/center, title='Fatal Error in 2D Slice Options')
      
      ; destroy window if state not valid
      widget_control, event.top, /destroy
    
    endelse
    
    return
  endif


;kill requests
  if tag_names(event,/struc) eq 'WIDGET_KILL_REQUEST' then begin
    thm_ui_slice2d_message,'Widget Killed',hw=state.historywin, /dontshow
    widget_control, event.top, set_uval=state, /no_copy
    widget_control, event.top, /destroy
    return
  endif


;use value for case statement
  widget_control, event.id, get_uval=uval

  if size(uval,/type) ne 0 and n_elements(uval) le 1 then begin
    Case uval Of

      'DONE':Begin
        thm_ui_slice2d_message,'2D Slice Options Closed',hw=state.historywin, sb=state.statusbar, /dontshow
        widget_control, event.top, set_uval=state, /no_copy
        widget_control, event.top, /destroy
        return
      end

      'GENERATE': begin
        widget_control, /hourglass
        thm_ui_slice2d_message,'Working...', sb=state.statusbar
        thm_ui_slice2d_gen, state.tlb, state
        store_data, tnames('*2dslice_temp*'), /delete
      end
      
      'REPLOT': begin
        state.slider->getProperty, value=current
        thm_ui_slice2d_plot, state, current
      end

      'ZOOM': begin ;experimenal feature
;        cursor, x0, y0, 3, /data
;        cursor, x1, y1, 4, /data
;        
;        xr = minmax([x0,x1])
;        yr = minmax([y0,y1])
;        thm_ui_slice2d_message, sb=state.statusbar, $
;          'Re-plotting from ('+strtrim(xr[0],2)+', '+strtrim(yr[0],2)+ $
;          ') to ('+strtrim(xr[1],2)+', '+strtrim(yr[1],2)+')'
;
;        state.slider->getProperty, value=current
;        thm_ui_slice2d_plot, state, current, xrange=[x0,x1], yrange=[y0,y1]
      end

      'SLIDERBAR': begin
        state.slider->update, /event
        thm_ui_slice2d_plot, state, event.value
      end

      'ROTLEGEND': thm_ui_slice2d_rotlegend, state.tlb, state.gui_id
      
      'MAG':
      
      'PROBE':
      
      'DTYPE':
      
      'UTYPE': 
      
      'VTYPE':
      
      'SLICETYPE': thm_ui_slice2d_sens, state, event.id, event.select

      'ERANGE': widget_control, state.erangebase, sens=event.select
      
      'THETARANGE': widget_control, state.thetarangebase, sens=event.select
      
      'ZDIRRANGE': widget_control, state.zdirrangebase, sens=event.select

      'SUBTRACT': thm_ui_slice2d_velsens, state
      
      'ROT': thm_ui_slice2d_velsens, state

      'ONECOUNT': ;widget_control, state.regridbase, sens= ~event.select
      
      'SSTCAL': state.flags.forcereload = 1b
      
      'SMOOTH': begin
        id = widget_info(event.top, find_by_uname='smoothbase')
        widget_control, id, sens=event.select
      end
      
      'REGRID': begin
        id = widget_info(event.top, find_by_uname='regridbase')
        widget_control, id, sens=event.select
      end
      
      'RES': begin
        id = widget_info(event.top, find_by_uname='resolution')
        widget_control, id, sens=event.select
        state.flags.resset = event.select
      end
      
      'CONTAMINATION': thm_ui_slice2d_contsens, event
      
      'SSTMETHOD': begin ;change tolerance value if set at defaults
        id = widget_info(event.top, find_by_uname='ssttolerance')
        widget_control, id, get_value=tol
        if (tol eq 2d) && (event.index gt 1) then $
          widget_control, id, set_value= 3.5d
        if (tol eq 3.5d) && (event.index lt 2) then $
          widget_control, id, set_value= 2d  
      end
      
      'ZMINMAX': widget_control, state.zminmaxbase, sens=~event.select
      
      'XYMINMAX': widget_control, state.xyminmaxbase, sens=~event.select
      
      'OLINES': begin
        id = widget_info(state.tlb, find_by_uname='nolines')
        widget_control, id, sens = event.select
        state.flags.olinebutton = 1b
      end
      
      'NOLINES': state.flags.olineset = 1b
      
      'XTICKS': thm_ui_slice2d_ticksens, event, uval

      'YTICKS':thm_ui_slice2d_ticksens, event, uval

      'ZTICKS':thm_ui_slice2d_ticksens, event, uval

      'EXPORTPNG': thm_ui_slice2d_export, state, uval
      
      'EXPORTEPS': thm_ui_slice2d_export, state, uval
      
      'EXPORTALLPNG': thm_ui_slice2d_export, state, uval, all=1b
      
      'EXPORTALLEPS': thm_ui_slice2d_export, state, uval, all=1b
      
      Else: dprint,  uval ;'Unkown Uval'
    EndCase
  endif

widget_control, event.top, set_uval=state, /no_copy

end ;----------------------------------------------------



;+
;NAME:
;  thm_ui_slice2d
;
;PURPOSE:
;  Front end window allowing user to create and view "2D" slices 
;  of velocity distributions.
;
;CALLING SEQUENCE:
;  thm_ui_slice2d [, gui_ID=gui_ID [, historywin=historywin]]
;
;INPUT:
;  gui_id: group leader widget if opening from THEMIS GUI
;  historywin: history window object if opening from THEMIS GUI
;;
;OUTPUT:
;  N/A  
;
;NOTES: For command line use see:
;         plot_part_slice2d.pro
;         thm_part_slice2d.pro
;         thm_part_dist_array.pro
;  
;-


pro thm_ui_slice2d, gui_ID=gui_ID, historywin=historywin

    compile_opt idl2

  tlb_title = 'Ion/Electron Velocity Slices v3.2'

if keyword_set(gui_ID) then begin
  tlb = widget_base(title = tlb_title, /col, /base_align_center, $ 
                    group_leader=gui_id, /modal, /tlb_kill_request_events, tab_mode=1)
endif else begin
  tlb = widget_base(title = tlb_title, /col, /base_align_center, $ 
                    /tlb_kill_request_events, tab_mode=1)
  gui_ID=tlb
endelse

;Ensure color table properly set
thm_init
thm_graphics_config

;Bases
;-----

;Tabs
  tabBase = widget_base(tlb, /col, xpad=0, ypad=0, space=2, /base_align_center)
  tabs = widget_tab(tabBase, /align_center)

;Main Base
  mainBase = widget_base(tabs, title='Main', /col, xpad=4, ypad=4, space=2)
    typerow = widget_base(mainbase, /row, /align_center, ypad=2, space=10)
    timebase = widget_base(mainbase, /col, /align_center, ypad=4, xpad=6, $
                           space=0, frame=1)
      dummy = widget_base(mainbase, ypad=6)
    slicetyperow = widget_base(mainbase, /row, /align_center, ypad=4, space=8)
    mainoptionsbase = widget_base(mainbase, /col, /align_left, ypad=2)
      moptbase1 = widget_base(mainoptionsbase, /col, space=4)

;Options Base
  optionsBase = widget_base(tabs, title='Options', /col, xpad=4, ypad=6, space=4)
    rangebase2dnn = widget_base(optionsBase, /col, /align_center, ypad=6, xpad=4)
    rangebase2d = widget_base(optionsBase, /col, /align_center, ypad=6, xpad=4)
;    rangebase3d = widget_base(optionsBase, /col, /align_center, ypad=6, xpad=4)
    optionsBase1 = widget_base(optionsBase, /col, /align_center, ypad=2)

;Contamination Base
  contaminationBase = widget_base(tabs, title='Contamination', /col, xpad=4, ypad=8, space=8)
    esaContBase = widget_base(contaminationBase, /col, /align_left, ypad=4, xpad=4)
    sstContBase = widget_base(contaminationBase, /col, /align_left, ypad=4, xpad=4)

;Plot Options Base
  plotBase = widget_base(tabs, title='Plot Options', /col, xpad=4, ypad=4, space=2)
    plotoptionsbase1 = widget_base(plotbase, /col, /align_left, ypad=4, xpad=4, space=4)
    plotoptionsbase2 = widget_base(plotbase, /row, /align_center, ypad=4, xpad=4, space=6)
    plotoptionsbase3 = widget_base(plotbase, /col, /align_center, ypad=1, xpad=1, space=2)
        
;Bottom Bases
  sliderbase = widget_base(tabbase, /col, /align_center, ypad=2, xpad=0)
  buttonBase = widget_base(tabbase, /row, /align_center, ypad=2, xpad=0)
  statusbarBase = widget_base(tabbase, /row, ypad=0, xpad=0)



;Bottom widgets
;--------------

;Slider bar
  slider = obj_new('thm_ui_slice2d_bar', sliderbase, 100, statusbar=statusbar)

  
;Buttons
  buttonsize=60
  generateplot = widget_button(buttonbase, value='Generate', uval='GENERATE', $
                               xsize=buttonsize*(1+sqrt(5))/2, tooltip=$
                               'Loads distributions, creates slices, and plots ' +$
                               'slice at center of given time range.')
  done = widget_button(buttonbase, value = 'Close', xsize=buttonsize, uval='DONE')
    dummy = widget_label(buttonbase, value = ' ',xsize=buttonsize*.5)
  exportslice = widget_button(buttonbase, value='Export', /menu)
    exportpng = widget_button(exportslice, value='PNG', uval='EXPORTPNG')
    exporteps = widget_button(exportslice, value='EPS', uval='EXPORTEPS')
  exportall = widget_button(buttonbase, value='Export All', /menu)
    exportallpng = widget_button(exportall, value='PNG', uval='EXPORTALLPNG')
    exportalleps = widget_button(exportall, value='EPS', uval='EXPORTALLEPS')

;Output
  statusbar = obj_new('thm_ui_message_bar', statusbarbase,  Xsize=60, YSize=1, $
                      value='Status information is displayed here.')
  if ~obj_valid(historywin) then begin
    historyWin = Obj_New('THM_UI_HISTORY', 0L, tlb);dummy history window in absence of gui
  endif



;Main Tab Widgets
;------------

;Data type selection

  probes = ['a','b','c','d','e']
  ctypes = ['DSL','GSE','GSM']
  dtypes = ['peif','peir','peib','psif','psir', $ ;no 'psib'
            'peef','peer','peeb','psef','pser','pseb']
  utypes = ['Counts','DF','Rate','Flux','EFlux']
  magtypes = ['fgl','fgh','fgs','fge']
  rotlong = ['BV', 'BE', 'xy', 'xz', 'yz','xvel', $  ;type 2 rotations 
             'perp', 'perp_xy', 'perp_xz', 'perp_yz']
  rotshort = ['BV', 'BE', 'xyz','xvel', 'perp', 'perp_xyz'] ;type 1 & 3 rotations 
  rotmag = ['BV', 'BE', 'perp', 'perp_xyz', $ ;rotations requiring mag data
            'perp_xy', 'perp_xz', 'perp_yz']
  rotvel = ['BV','BE','xvel','perp'] ;rotations requiring vel data

  probebase = widget_base(typerow, /col)
    probelabel = widget_label(probebase, value='Probe: ')
    probe = widget_list(probebase, value=probes, uval=probes, $
                        uname='probe', ysize=5)
  disttypebase = widget_base(typerow, /col)
    disttypelabel = widget_label(disttypebase, value='Distribution Type:')
    disttype = widget_list(disttypebase, uname='dtype', uval=dtypes, $
                  ysize=n_elements(dtypes), value=dtypes, /multiple)
  unittypebase = widget_base(typerow, /col)
    unittypelabel = widget_label(unittypebase, value='Units: ')
    unittype = widget_list(unittypebase, uname='utype', uval=utypes, $
                  ysize=n_elements(utypes), value=utypes)

  coordtypebase = widget_base(unittypebase, /col, ypad=4)
    coordtypelabel = widget_label(coordtypebase, value='Coord: ')
    coordtype = widget_list(coordtypebase, uname='ctype', uval=ctypes, $
                  ysize=n_elements(ctypes), value=ctypes)
                  
    slicetypebase = widget_base(slicetyperow, /exclusive, /row, xpad=8, ypad=0, space=0)
      buttonGeo = widget_button(slicetypebase, value='Geometric', $
                               uname='buttongeo', uval='SLICETYPE', $
                               tooltip='Points on the slice plane are assigned values '+ $
                               'by which bin(s) they intersect. Use high resolution to '+ $
                               'view bin boundaries and low resolution for smooth gradients.')
      button2di = widget_button(slicetypebase, value='2D Interpolation', $
                               uname='button2di', uval='SLICETYPE', $
                               tooltip='Datapoints within the specified theta or z-axis range '+ $
                               'are projected onto the slice plane and linearly interpolated.')
      button3di = widget_button(slicetypebase, value='3D Interpolation', $
                               uname='button3di', uval='SLICETYPE', $
                               tooltip='The distribution is linearly interpolated in '+ $
                               'three dimensions then a slice is extracted along the '+ $
                               'designated orientation. (May interpolate over gaps in data).')
;      button2dnn = widget_button(slicetypebase, value='2D Nearest Neighbor', $
;                               uname='button2dnn', uval='SLICETYPE', $
;                               tooltip='A slice of data with user defined width is projected '+ $
;                               'onto the slice plane and interpolated onto a regular grid '+ $
;                               'using the nearest neighbor.')


;Time range widgets

  cad = '30'       ;seconds
  incriment = '30' ;

  time = thm_ui_time_widget(timebase, statusbar, historywin, uname='time', suppressoneday=1)

  timeoptbase = widget_base(timebase, /row, xpad=3)
    timewindow = thm_ui_spinner(timeoptbase, label='Window Size (sec): ',  $
                        value=cad, uname='timewin', text_box_size=6, incr=1, $
                        getxlabelsize=timelabelsize, min_value=1, $
                        tooltip='Window over which data will be averaged.')
    padbase = widget_base(timeoptbase, /row)
    cBase = widget_base(timeoptbase, /nonexclusive, xpad=0, ypad=0, space=0)
      center = widget_button(cBase, value='Center Time', uname='center', $
                             tooltip='Specified times are used as the center of the time window.')
  timeoptbase1 = widget_base(timebase, /row, xpad=3)
    timeinc = thm_ui_spinner(timeoptbase1, label='Step Time (sec): ', $
                        value=incriment, uname='timeinc', text_box_size=6, incr=1, $
                        xlabelsize=timelabelsize, min_value=1, $
                        tooltip='Time step between slices.')


;Main options

  orn = ['0','0','1'] ;default orienation vector
  ndisplacement = '0' ;km/s
  vels = ['Auto','peif','peib','peir','peef','peeb','peer','peim','peem','peim (mag)','peem (mag)']

  rotbase = widget_base(moptbase1, /row, xpad=0, ypad=0)
    rotationlabel = widget_label(rotbase, value='Rotation: ')
    rotation = widget_combobox(rotbase, uname='rot', uval='ROT', $
                               value=rotshort)
    rotlegend = widget_button(rotbase, value=' ? ', uval='ROTLEGEND', tooltip = $
                              'Displays description of available rotations')
  
  velbase = widget_base(moptbase1, /row, xpad=0, ypad=0)
    vellabel = widget_label(velbase, value='Bulk Velocity, & Mag Data: ')
      mgeo = widget_info(vellabel,/geo)
    veltype = widget_combobox(velbase, uname='vtype', uval='VTYPE', $
                               value=vels)
    magdata = widget_combobox(velbase, uname='mag', uval='MAG', $
                              value=magtypes)
  
  orientationbase = widget_base(moptbase1, /row, xpad=0, ypad=0)
    orientationlabel = widget_label(orientationbase, value='Slice Plane Normal (x,y,z): ')
    orx = widget_text(orientationbase, value=orn[0], xsize=4, uname='orx', /edit)
    ory = widget_text(orientationbase, value=orn[1], xsize=4, uname='ory', /edit)
    orz = widget_text(orientationbase, value=orn[2], xsize=4, uname='orz', /edit)

  displacementbase = widget_base(moptbase1, /row, xpad=0, ypad=0)
    displacement = thm_ui_spinner(displacementbase, text_box_size=6, incr=50, $
                              uname='displace', label='Displacement(km/s): ', $
                              xlabelsize=mgeo.scr_xsize-1, value=ndisplacement, $
                              tooltip='Displacement of slice from zero along '+ $
                              "the slice plane's normal")




;Options Tab Widgets
;--------------------

  nthetamin = '-20' ;degrees
  nthetamax = '20'
  nzdirmin = '-250' ;km/s
  nzdirmax = '250'
  nemin = '0'  ;eV
  nemax = '30000'
  smth = '3'  ;smoothing width (# points)
  regrid = ['64','32','64']  ;regrid defaults
  res = '500' ;resolution
  wth = '4' ;slice width (%)

;2D Nearest Neighbor ranges

;Obsolete (keep for testing)
;  label2dnn = widget_label(rangebase2dnn, value='2D Nearest Neighbor Limits')
;  subrangebase2dnn = widget_base(rangebase2dnn, /col, /base_align_center, $
;                                 xpad=4, ypad=4, frame=1)
;    widthbase = widget_base(subrangebase2dnn, /row)
;      width = thm_ui_spinner(widthbase, value=wth, text_box_size=6, incr=1, $
;                             uname='slice_width', label='Slice Width (%):   ', $
;  ;                           xlabelsize=geo.scr_xsize-1, $
;                             tooltip = $
;                             'Entire width of the slice about the normal as'+ $
;                             ' a percentage of the total range')

;2D Interpolation Ranges

  label2d = widget_label(rangebase2d, value='2D Interpolation Limits')
  subrangebase2d = widget_base(rangebase2d, /col, /base_align_center, $
                            xpad=4, ypad=4, frame=1)
    buttons2dbase = widget_base(subrangebase2d, /row, /exclusive)
      thetarangebutton = widget_button(buttons2dbase, value='Restrict Latitude', $
                                 uname='thetarange', uval='THETARANGE')
      zdirrangebutton = widget_button(buttons2dbase, value='Restrict Z-Axis', $
                                 uname='zdirrange', uval='ZDIRRANGE')
    range2dbase = widget_base(subrangebase2d, /row)
      thetarangeBase = widget_base(range2dbase, /col, xpad=4, ypad=0)
        thetamax = thm_ui_spinner(thetarangeBase, label='Max ('+string(176b)+'): ', $
                                value=nthetamax, uname='thetamax', text_box_size=7, $
                                incr=1, getxlabelsize=scr_xsize, max_value=90, min_value=-90)
        thetamin = thm_ui_spinner(thetarangeBase, label='Min ('+string(176b)+'): ', $
                                value=nthetamin, uname='thetamin', text_box_size=7, $
                                incr=1, xlabelsize=scr_xsize, max_value=90, min_value=-90)
          dummy = widget_label(range2dbase, value=' ')
      zdirrangeBase = widget_base(range2dbase, /col, xpad=4, ypad=0)
        zdirmax = thm_ui_spinner(zdirrangeBase, label='Max (km/s): ', $
                                value=nzdirmax, uname='zdirmax', text_box_size=7, $
                                incr=1, getxlabelsize=scr_xsize)
        zdirmin = thm_ui_spinner(zdirrangeBase, label='Min (km/s): ', $
                                value=nzdirmin, uname='zdirmin', text_box_size=7, $
                                incr=1, xlabelsize=scr_xsize)

;General Slice ranges

  label3d = widget_label(optionsBase1, value='General Slice Options')
  optionsSubBase1 = widget_base(optionsBase1, /col, xpad=0, ypad=0, frame=1)
  
  subrangebase3d = widget_base(optionsSubBase1, /col, /base_align_center, $
                            xpad=4, ypad=4)
    erangebase = widget_base(subrangebase3d, /row, /base_align_center, $
                              xpad=0, ypad=1, space=0)
      erangeBase1 = widget_base(erangebase, /row, xpad=0, ypad=0, /nonexclusive)
        erangebutton = widget_button(erangebase1, value='Restrict Energy Range', $
                                   uname='erange', uval='ERANGE') 
      erangeBase2 = widget_base(erangebase, /col, xpad=4, ypad=0)
        emax = thm_ui_spinner(erangebase2, label='Max (eV): ', value=nemax, $
                                  uname='emax', text_box_size=8, incr=1, $
                                  getxlabelsize=scr_xsize, min_value=10)
        emin = thm_ui_spinner(erangebase2, label='Min (eV): ', value=nemin, $
                                  uname='emin', text_box_size=8, incr=1, $
                                  xlabelsize=scr_xsize, min_value=0)

;Other Options
  optbase1 = widget_base(optionsSubBase1, /col, /align_left, space=2, xpad=4, ypad=4)

    regridbase = widget_base(optbase1, /row, xpad=0, ypad=0)
      regridbuttonbase = widget_base(regridbase, /row, xpad=0, ypad=0, /nonexclusive)
        regridbutton = widget_button(regridbuttonbase, $
                         value='Regrid Data (phi, theta, energy):', uval='REGRID', uname='regrid', $
                         tooltip='Spherically interpolates values at each energy onto a '+ $
                                 'new grid with N x M x L points in phi and theta and '+ $
                                 'energy respectively. (Uses nearest neighbor)')
      regridsubbase = widget_base(regridbase, /row, xpad=0, ypad=0, uname='regridbase')
        regridphi = widget_text(regridsubbase, value=regrid[0], xsize=4, uname='regridphi', /edit)
        regridtheta = widget_text(regridsubbase, value=regrid[1], xsize=4, uname='regridtheta', /edit)
        regride = widget_text(regridsubbase, value=regrid[2], xsize=4, uname='regride', /edit)

    smoothbase = widget_base(optbase1, /row, xpad=0, ypad=0)
      smoothbuttonbase = widget_base(smoothbase, /row, xpad=0, ypad=0, /nonexclusive)
        smoothbutton = widget_button(smoothbuttonbase, value='Smooth Data (width)', uname='smooth', $
                            uvalue='SMOOTH', tooltip='Applies boxcar average to slice.')
      smoothsubbase = widget_base(smoothbase, /row, xpad=0, ypad=0, uname='smoothbase')
        smoothspinner = thm_ui_spinner(smoothsubbase, value=smth, text_box_size=4, incr=2, $
                            uname='smooth_width', label='', min_value=3, tooltip = 'Width of '+ $
                            'the smoothing window in # of points (only use odd integers >=3)')

    suboptbase1 = widget_base(optbase1, /col, xpad=0, ypad=0, /nonexclusive)
      subtract = widget_button(suboptbase1, value='Subtract Bulk Velocity', $
                               uname='subtract', uval='SUBTRACT', tooltip = $
                               'Subtracts specified bulk velocity from distribution.')      
      onecount = widget_button(suboptbase1, value='Remove bins below one-count level', $
                              uname='onecount', uval='ONECOUNT', tooltip= $
                              'After averaging, remove bins with values below the one-count level.')
      sstcal = widget_button(suboptbase1, value='Use new SST calibrations', $
                             uname='sstcal', uval='SSTCAL', tooltip='See documentation in <?>.')
    resolutionbase = widget_base(optbase1, /row, xpad=0, ypad=0)
      resbuttonbase = widget_base(resolutionbase, /row, xpad=0, ypad=0,/nonexclusive)
        resbutton = widget_button(resbuttonbase, value='Specify Resolution: ', $
                            uvalue='RES', tooltip='Specfies the number of elements in each '+ $
                            'dimension of the slice.')
      resbase = widget_base(resolutionbase, /row, xpad=0, ypad=0)
        resolution = thm_ui_spinner(resbase, value=res, text_box_size=6, uname='resolution', $
                                 label='  ', incr=10, min_value=10, $ ;xlabelsize=geo.scr_xsize-1, $
                                 tooltip = 'Specfies the number of elements in each '+ $
                                 'dimension of the slice.')



;Contamination Tab
;-------------------
  esaBgndTypes = ['Anode','Angle','Omni'] ;types of background removal for esa (bgnd_type)
  esaNPoints = 3.0 ; bgnd_npoints 
  esaScaleVal = 1.0 ; bgnd_scale
  sstMaskVal = 0.999 ;mask_remove
  sstMethods = ['median','spinfit','z_score_mod'] ;method_sunpulse_clean
  sstToleranceLimits = [1.8,3.5] ; limit_sunpulse_clean 
                                 ; 3.5 is default for zmod, 2 for median/spinfit
  sstFillMethods = ['Interpolation','Spin Fit'] ; fillin_method  
  
  ;ESA Widgets
  esaRemoveButtonbase = widget_base(esaContBase, /row, /nonexclusive)
    esaRemoveButton = widget_button(esaRemoveButtonbase, val='Remove ESA Background', $
                                    uval='CONTAMINATION', uname='esaremove')
  
  esaRemoveOptsBase = widget_base(esaContBase, /col, xpad=12, uname='esaremovebase', frame=1)
    esaTypeBase = widget_base(esaRemoveOptsBase, /row)
      esaTypeLabel = widget_label(esaTypeBase, value='Type:  ')
      esaType = widget_combobox(esaTypeBase, value=esaBgndTypes, uname='esatype')
    esaNPtsBase = widget_base(esaRemoveOptsBase, /row)
      esaNPts = thm_ui_spinner(esaNPtsBase, label='Number of Points:  ', text_box_size=6, $
                               uname='esanpoints', incr=1, value=esaNPoints, $
                               getXLabelSize=scr_xsize, min_value=1, $
                               tooltip='The number of lowest values points to '+ $
                               'average over when determining background.')
      widget_control, esaTypeLabel, xsize = scr_xsize+1
    esaScaleBase = widget_base(esaRemoveOptsBase, /row)
      esaScale = thm_ui_spinner(esaScaleBase, label='Scaling:  ', text_box_size=6, $ 
                                uname='esascale', incr=1, value=esaScaleVal, $
                                xlabelsize=scr_xsize, min_value=0, $
                                tooltip='A scaling factor that the background '+ $
                                'will be multiplied by before it is subtracted.')
  
  ;SST Widgets
  sstRemoveMaskBase = widget_base(sstContBase, /row, ypad=4)
    sstRemoveMaskButtonBase = widget_base(sstRemoveMaskBase, /row, /nonexclusive, xpad=0)
      sstRemoveMaskButton = widget_button(sstRemoveMaskButtonBase, val='Remove SST Mask', $
                                    uval='CONTAMINATION', uname='sstmaskremove', $
                                    tooltip='The proportion of values that must be 0 '+ $
                                   'at all energies to determine that a mask is present.')
    sstRemoveMask = thm_ui_spinner(sstRemoveMaskBase, val=sstMaskVal, text_box_size=8, $
                                   uname='sstmask', incr=0.01, min_value=0.01, max_value=1, $
                                   tooltip='the proportion of values that must be 0 '+ $
                                   'at all energies to determine that a mask is present.')
  
  sstRemoveButtonBase = widget_base(sstContBase, /row, /nonexclusive)
    sstRemoveButton = widget_button(sstRemoveButtonBase, val='Remove SST Contamination', $
                                    uval='CONTAMINATION', uname='sstremove')

  sstRemoveOptsBase = widget_base(sstContBase, /col, xpad=12, uname='sstremovebase', frame=1)
    sstMethodBase = widget_base(sstRemoveOptsBase, /row)
      sstMethodLabel = widget_label(sstMethodBase, value='Method:  ')
      sstMethod = widget_combobox(sstMethodBase, val=sstMethods, uname='sstmethod', $
                                  uval='SSTMETHOD')
    sstFillBase = widget_base(sstRemoveOptsBase, /row)
      sstFillLabel = widget_label(sstFillBase, value='Fill Method:  ')
      sstFill = widget_combobox(sstFillBase, val=sstFillMethods, uname='sstfill')
    sstToleranceBase = widget_base(sstRemoveOptsBase, /row)
      sstToleranceLabel = widget_label(sstToleranceBase, value='Tolerance (stdv):  ')
      sstTolerance = thm_ui_spinner(sstToleranceBase, val=sstToleranceLimits[0], $
                                    uname='ssttolerance', incr=0.1, min_value=0.1, $
                                    tooltip='Tolerance in standard deviations '+ $
                                    'for the given method.')
      sgeo = widget_info(sstToleranceLabel,/geo)
      widget_control, sstMethodLabel, xsize = sgeo.scr_xsize
      widget_control, sstFillLabel, xsize = sgeo.scr_xsize


  

;Plot Tab Widgets
;--------------------
  nolines = '8'  ;# contour lines
  nlevels = '60'  ;# color countour levels
  zmax = '1e1'  ; plotting limites (various units)
  zmin = '1e-6'
  xymin = '-2000'
  xymax = '2000'
  nxticks = '4'  ;tick numbers
  nyticks = '4'
  nzticks = '11'
  ncharsize = '100' ;character size (%)
  nprecision = ['1234 (rounded down)', '1.', '1.2', '1.23', '1.234', '1.1234', $
                '1.01234', '1.001234','1.0001234','1.00001234','1.000001234']

  moreoptionsbase = widget_base(plotoptionsbase1, /col, /base_align_center, space=2)
    numlevels = thm_ui_spinner(moreoptionsbase, label='Number of Color Contour Levels: ', $
                             uname='nlevels', text_box_size=5, incr=1, value=nlevels, $
                             min_value=1, tooltip='Specifies the number of color contours to draw.')

  olinesbase = widget_base(plotoptionsbase1, /row, /base_align_center, space=2)
    olinebase1 = widget_base(olinesbase, /col, xpad=0, ypad=0, /nonexclusive)
      olinebutton = widget_button(olinebase1, value='Plot Contour Lines', uval='OLINES', $
                         uname='olines',tooltip='Draws contour lines over default plot.')
    numolines = thm_ui_spinner(olinesbase, label='Number of Contour Levels: ', uval='NOLINES', $
                          uname='nolines', text_box_size=5, incr=1, value=nolines, $
                          min_value=1, tooltip='Specifies the number of contour lines to draw.')

  minmaxbase = widget_base(plotoptionsbase1, /row, /base_align_center, space=0)
    minmaxBase1 = widget_base(minmaxbase, /col, xpad=0,ypad=0, /nonexclusive)
      minmaxbutton = widget_button(minmaxBase1, value='Autorange (Z)', $
                                 uname='zminmax', uval='ZMINMAX', tooltip = $
                                 'Non-zero, pre-interpolated extremes used as range. '+ $
                                 '(Values below this minimum could be the result '+ $
                                 'of interpolation/smoothing)') 
    minmaxBase2 = widget_base(minmaxbase, /col, xpad=4, ypad=0)
        dummy2 = widget_label(minmaxBase2, value='Max (units): ')
        geo2 = widget_info(dummy2, /geo)
        widget_control, dummy2, /destroy
      zmaximum = thm_ui_spinner(minmaxbase2, label='Max (units): ', value=zmax, $
                                uname='zmax', text_box_size=10, incr=1, $
                                xlabelsize=geo2.scr_xsize)
      zminimum = thm_ui_spinner(minmaxbase2, label='Min (units): ', value=zmin, $
                                uname='zmin', text_box_size=10, incr=1, $
                                xlabelsize=geo2.scr_xsize)

  xyminmaxbase = widget_base(plotoptionsbase1, /row, /base_align_center, space=0)
    xyminmaxBase1 = widget_base(xyminmaxbase, /col, xpad=0,ypad=0, /nonexclusive)
      xyminmaxbutton = widget_button(xyminmaxBase1, value='Autorange (X,Y)', $
                                 uname='xyminmax', uval='XYMINMAX', tooltip = $
                                 'Grid extremes used as default. ') 
    xyminmaxBase2 = widget_base(xyminmaxbase, /col, xpad=4, ypad=0)
        dummy2 = widget_label(xyminmaxBase2, value='Max (km/s): ')
        geo2 = widget_info(dummy2, /geo)
        widget_control, dummy2, /destroy
      xymaximum = thm_ui_spinner(xyminmaxbase2, label='Max (km/s): ', value=xymax, $
                                uname='xymax', text_box_size=10, incr=1, $
                                xlabelsize=geo2.scr_xsize)
      xyminimum = thm_ui_spinner(xyminmaxbase2, label='Min (km/s): ', value=xymin, $
                                uname='xymin', text_box_size=10, incr=1, $
                                xlabelsize=geo2.scr_xsize)

  checkboxtopbase = widget_base(plotoptionsbase1, /row, space=8, ypad=0, xpad=0)
    checkboxbase1 = widget_base(checkboxtopbase, /col, /nonexclusive, space=2)
      logbutton = widget_button(checkboxbase1, value='Logarithmic Plot', $
                                uname='logplot', tooltip = $
                                'Create logarithmic contour lines and color gradient.')
      sundirbutton = widget_button(checkboxbase1, value='Plot Sun Direction', $
                                   uname='sundir', tooltip = $
                                   'Plot slice plane projection of sun vector.')
      bulkbutton = widget_button(checkboxbase1, value='Plot Bulk Velocity', $
                                   uname='plotbulk', tooltip = $
                                   'Plot slice plane projection of bulk velocity.')
    checkboxbase2 = widget_base(checkboxtopbase, /col, /nonexclusive, space=2)
      axesbutton = widget_button(checkboxbase2, value='Plot Axes', $
                                   uname='axes', tooltip = $
                                   'Plots dashed lines along zero for both axes.')
      energycircbutton = widget_button(checkboxbase2, value='Draw Energy Limits', $
                                       uname='ecirc', tooltip = $
                                       'Plots upper/lower velocity (energy) limits.')
      labelcontours = widget_button(checkboxbase2, value='Label Contour Lines', $
                                    uname='labelcontours', tooltip = $
                                    'Plots the numerical value of each contour line')


;Ticks and annotations

  textbase = widget_base(plotoptionsbase2, /col, /base_align_left)
    annotatelabel = widget_label(textbase, value='Annotations  ')
    annotatebase = widget_base(textbase, /col, xpad=4, ypad=6, space=6, frame=1)
      precisionbase = widget_base(annotatebase, /row, xpad=0, ypad=0)
        precisionlabel = widget_label(precisionbase, value='Precision: ')
        precision = widget_combobox(precisionbase, value=nprecision, uname='precision')
      charsize = thm_ui_spinner(annotatebase, label='Size: ', uname='charsize', $
                                xlabelsize=(widget_info(precisionlabel,/geo)).scr_xsize-1, $
                                text_box_size=5, incr=5, value=ncharsize, units='%',$
                                min_value=0, tooltip='Size of the text printed on the plot')
      astylebase =  widget_base(annotatebase, /col, /exclusive, ypad=0, xpad=0)
        annodefault = widget_button(astylebase, value = 'Automatic', uname='anno0', $
                             tooltip='Auto-selects between decimal and scientific notation')
        annodecimal = widget_button(astylebase, value = 'Decimal Notation', uname='anno1')
        annoexpo = widget_button(astylebase, value = 'Scientific Notation', uname = 'anno2') 

  ticknumbase = widget_base(plotoptionsbase2, /col, /base_align_left)
    ticknumlabel = widget_label(ticknumbase, value='Specify Number of Ticks  ') 
    ticknumbase1 = widget_base(ticknumbase, /col, xpad=0, ypad=0, frame=1);4,6,1
      ticknumsubbase1 = widget_base(ticknumbase1, /row, space=0,$;space=6
                                    /align_right, xpad=2,ypad=6);/nonexclusive
      ticknumsubbase2 = widget_base(ticknumbase1, /row, space=0, xpad=2, ypad=6)
      ticknumsubbase3 = widget_base(ticknumbase1, /row, space=0, xpad=2, ypad=6)
        ticknumsubbase1button = widget_base(ticknumsubbase1,/col,/nonexclusive,/align_right)                            
        ticknumsubbase2button = widget_base(ticknumsubbase2,/col,/nonexclusive,/align_right)
        ticknumsubbase3button = widget_base(ticknumsubbase3,/col,/nonexclusive,/align_right)
       
        ticknumbuttonx = widget_button(ticknumsubbase1button,value=' ', $
            uval='XTICKS', tooltip='Specifies number of ticks to draw on the x-axis.')
        ticknumbuttony = widget_button(ticknumsubbase2button, value=' ', $
            uval='YTICKS', tooltip='Specifies number of ticks to draw on the y-axis.')
        ticknumbuttonz = widget_button(ticknumsubbase3button, value=' ', $
            uval='ZTICKS', tooltip='Specifies number of ticks to draw on z-axis.')

        ticknumx = thm_ui_spinner(ticknumsubbase1, label='X-Axis: ', sens=0, $
                          uname='nxticks', text_box_size=5, incr=1, value=nxticks, $
                          min_value=0, max_value=60, tooltip='Number of ticks to place on the X-Axis.')
        ticknumy = thm_ui_spinner(ticknumsubbase2, label='Y-Axis: ', sens=0, $
                          uname='nyticks', text_box_size=5, incr=1, value=nyticks, $
                          min_value=0, max_value=60, tooltip='Number of ticks to place on the Y-Axis.')
        ticknumz = thm_ui_spinner(ticknumsubbase3, label='Z-Axis: ', sens=0, $
                          uname='nzticks', text_box_size=5, incr=1, value=nzticks, $
                          min_value=0, max_value=60, tooltip='Number of ticks to place on the Z-Axis bar.')
  
;Other

  plotbuttonbase = widget_base(plotoptionsbase3, /row, /base_align_center)
    replot = widget_button(plotbuttonbase, value = 'Re-Plot', xsize=buttonsize, $
                         uval='REPLOT', tooltip='Replot current slice')
;    zoombutton = widget_button(plotbuttonbase, value='Zoom', xsize=buttonsize, sens=0, $
;                         uval='ZOOM', tooltip='Left click on plot to zoom.')




;Initialize widgets
;-----------------

; Main Widgets
  widget_control, unittype, set_list_select=1 ;default units (DF)
  widget_control, coordtype, set_list_select=0 ;default coords (DSL) 
  
  widget_control, smoothbutton, set_button=0 ; smoothing off by default
    widget_control, smoothsubbase, sens=0
  
  widget_control, regridbutton, set_button=0 ; regrid on by default
    widget_control, regridsubbase, sens=0

  widget_control, resbutton, set_button=0 ;auto res by default
    widget_control, resolution, sens=0

  widget_control, buttonGeo, set_button=1 ; type of slice
;  widget_control, button2dnn, sens=0 ; 2d nn for testing

  widget_control, thetarangebutton, set_button=1 ; default is to cut by theta
    widget_control, zdirrangeBase, sens=0

  widget_control, rotation, set_combobox_select=2 ; default xyz rotation

  widget_control, erangebase2, sens=0 ; energy limits off
  
  widget_control, orz, set_value='1' ;default orientation (0,0,1)

;Contamination Widgets
  widget_control, esaRemoveOptsBase, sens=0 ; off by def
  
  widget_control, sstRemoveMask, sens=0 ; off by def
  
  widget_control, sstRemoveOptsBase, sens=0 ; off by def

;Plot Widgets
  widget_control, olinebutton, set_button=0 ;contour lines off
    widget_control, numolines, sens=0 

  widget_control, logbutton, set_button=1
  widget_control, sundirbutton, set_button=0 ;may require STATE data
  widget_control, bulkbutton, set_button=1
  widget_control, axesbutton, set_button=1
  widget_control, energycircbutton, set_button=1

  widget_control, minmaxbutton, set_button=1 ; autorange by default
    widget_control, minmaxbase2, sens=0

  widget_control, xyminmaxbutton, set_button=1 ; autorange by default
    widget_control, xyminmaxbase2, sens=0

  widget_control, precision, set_combobox_select = 3 ;3 sig figs
  widget_control, annodefault, set_button=1


;Resize bases and widgets dynamically
;Most resizing (other than labels/spinners) will be done here
  widget_control, orientationlabel, xsize=mgeo.scr_xsize
  widget_control, rotationlabel, xsize=mgeo.scr_xsize

  zgeo = widget_info(olinebutton,/geo)
    widget_control, minmaxbutton, xsize=zgeo.scr_xsize - 3
    widget_control, xyminmaxbutton, xsize=zgeo.scr_xsize - 3

  ogeo = widget_info(regridbutton,/geo)
    widget_control, smoothbutton, xsize=ogeo.scr_xsize

  ;main widgets
  factor = 0.98
  geo = widget_info(tlb, /geo)
  widget_control, tlb, xsize = geo.scr_xsize * (1./factor)
;  widget_control, mainbase, xsize = geo.scr_xsize * factor
;  widget_control, optionsBase, xsize = geo.scr_xsize * factor
;  widget_control, contaminationBase, xsize = geo.scr_xsize * factor
;  widget_control, plotbase, xsize = geo.scr_xsize * factor

 
;Prep slider bar
  slider->setProperty, xsize = geo.scr_xsize * factor * 0.87
  slider->update


;Set time (arbitrary)
  t0 = '2008-02-26/04:54:00'
  t1 = '2008-02-26/04:54:30'
;  t0 = '2010-04-01/03:03:00'
;  t1 = '2010-04-01/03:04:00'
;  t0 = '2009-02-09/08:10:00'
;  t1 = '2009-02-09/08:20:00'
;  t0 = '2009-10-02/16:00:00'
;  t1 = '2009-10-02/16:05:00'
;  t0 = '2011-05-09/14:18:30'
;  t1 = '2011-05-09/14:19:00'
;  t0 = '2011-05-29/15:40:41'
;  t1 = '2011-05-29/15:40:45'
;  t0 = '2011-10-01/12:40:00'
;  t1 = '2011-10-01/13:00:00'
;  t0 = '2012-02-8/09:00:00'
;  t1 = '2012-02-8/12:00:00'
  tr = obj_new('thm_ui_time_range')
  x = tr->SetStartTime(t0)
  y = tr->SetEndTime(t1)
  widget_control, time, set_value=tr  



;Other Preparations
;------------------

  ; Make sure to also edit the error catch block in the event handler 
  ; if changing the following structure:
  flags = {forcereload:1b,olineset:0b,olinebutton:0b,resset:0b}
  

  ; Make sure to also edit the error catch block in the event handler 
  ; if changing the following structure:
  previous = {probeidx:-1,didx:-1,mag:'',vtype:'',trange:[-1d,-1d], $
              bgnd_remove:-1, bgnd_type:'', bgnd_npoints:-1d, bgnd_scale:-1d, $
              method_sunpulse_clean:'', limit_sunpulse_clean:-1d, $
              fillin_method:'', mask_remove:-1d, $
              esa_remove:-1, sst_remove:-1, sst_mask:-1}
              

  ;State info structure
  state = {tlb:tlb, gui_id:gui_id, historywin:historywin, statusbar:statusbar, $
            tlb_title:tlb_title, $
            erangebase:erangebase2, $
            zminmaxBase:minmaxBase2, xyminmaxBase:xyminmaxBase2, $
            thetarangebase:thetarangeBase, zdirrangebase:zdirrangebase, $
            rangebase2d:subrangebase2d, orientationbase:orientationbase, $
            displacementbase:displacementbase, $
            regridbase:regridbase, $ ;width:width, $
            slider:slider, $
            rotshort:rotshort, rotlong:rotlong, $
            rotvel:rotvel, rotmag:rotmag, $
            velbase:veltype, magbase:magdata, times:ptr_new(), $
            distribution:make_array(4,/ptr), slices:ptr_new(), $
            previous:previous, flags:flags, last:''}

  thm_ui_slice2d_sens, state, buttongeo, 1b, initpanel=1
  thm_ui_slice2d_velsens, state  

  centertlb, tlb
  widget_control, tlb, set_uvalue = state, /no_copy
  widget_control, tlb, /realize

  ;keep windows in X11 from snaping back to 
  ;center during tree widget events 
  if !d.NAME eq 'X' then begin
    widget_control, tlb, xoffset=0, yoffset=0
  endif

  xmanager, 'thm_ui_slice2d', tlb, /no_block

  heap_gc ; clean up memory before exit

  Return

end