pro eclipse_fgm_fit,probe=probe,eclipse_trange=eclipse_trange

; Add some padding around the eclipse times 
load_trange=[eclipse_trange[0] - 300.0D, eclipse_trange[1] + 300.0D]

; Load some mom velocity data, for before/after comparisons
thm_load_mom,probe=probe,trange=load_trange,level=1,type='calibrated',true_dsl=1,use_eclipse_corrections=1,suffix='_before'

; Load FGL data for refining delta_phi corrections
fgl_name='th'+probe+'_fgl_before'
thm_load_fgm,probe=probe,trange=load_trange,level=1,datatype='fgl',type='calibrated',use_eclipse_corrections=1,suffix='_before'
get_data,fgl_name,index=fgl_idx
if (fgl_idx EQ 0) then begin
   dprint,'FGL data not available during this eclipse, no offset generated.'
   return
end

; Load FGS data for refining delta_phi corrections. The difference between
; FGS and FGL spinplane angles represents the needed correction.  We
; are looking for a single offset value for the whole eclipse -- it
; doesn't need to be calculated on a point-by-point basis, which would
; introduce unwanted noise into the corrected FGS and MOM data.
;

fgs_name='th'+probe+'_fgs_before'
thm_load_fit,probe=probe,trange=load_trange,level=1,type='calibrated',datatype='fgs',true_dsl=1,use_eclipse_corrections=1,suffix='_before'
get_data,fgs_name,index=fgs_idx
if (fgs_idx EQ 0) then begin
   dprint,'Spin fit (FGS) data not available during this eclipse, no offset generated.'
   return
end

; Project the FGS and FGL B vectors onto the spin plane, then determine
; the angular difference between them.

spinplane_angle,fgs_name,output='fgs_angle',/degrees
spinplane_angle,fgl_name,output='fgl_angle',/degrees

calc,"'fgl_fit_offset' = 'fgl_angle' - 'fgs_angle'",/interpolate

; Put the offset values into the range [-180,180] degrees.

get_data,'fgl_fit_offset',data=d
idx=where(d.y LT -180.0, count)
if (count GT 0) then d.y[idx] = d.y[idx] + 360.0

idx=where(d.y GT 180.0, count)
if (count GT 0) then d.y[idx] = d.y[idx] - 360.0

store_data,'fgl_fit_offset',data=d

idx=where((d.x GE eclipse_trange[0]) AND (d.x LE eclipse_trange[1]),count)

if (count LT 100) then begin
   dprint,"Insufficient overlapping FGS and FGL data to determine offset."
   return
end

eclipse_offsets=d.y[idx]

print,string(n_elements(d.x)) + ' total offset points, ' + string(count) + ' in eclipse'

; Form a histogram of the FGS-FGL offset angles.  It should have a strong,
; narrow peak, with a bit of noise in the other bins.  We take all the values
; that were within a narrow range of the peak angle, discard everything
; else (noise), then average the remaining samples down to a single offset
; value to be applied for the whole eclipse.


binsize=1.0
hist=histogram(eclipse_offsets,min=-180.0,max=180.0,binsize=binsize,/nan)
peakcount=max(hist)
peakbin=where(hist EQ peakcount)

; We use peakbin[0] below, instead of peakbin, to force a scalar result,
; otherwise the "where" test won't work properly.

peak_offset = -180.0 + peakbin[0]*binsize + 0.5
print,'Peak count:' + string(peakcount) + ' Bin: ' + string(peakbin) + ' Angle: ' + string(peak_offset)

; Find all the measurements within "tolerance" degrees of the peak offset,
; and average them to determine the final correction.

tolerance = 2.5D
lower=peak_offset-tolerance
upper=peak_offset+tolerance
idx=where((eclipse_offsets GE lower) AND (eclipse_offsets LE upper),count)
if (count EQ 0) then begin
   message,'Internal error -- no samples found within tolerance of peak offset.'
end

final_offset=average(eclipse_offsets[idx])
print, "Final offset:", final_offset," degrees."

plot_names=[fgs_name,fgl_name]
;mom_vel_names=tnames('*velocity_before',mom_vel_count)
;if (mom_vel_count GT 0) then begin
;   plot_names=[plot_names,mom_vel_names]
;end

plot_names=[plot_names,'fgl_fit_offset']
tplot,plot_names,trange=load_trange
print,'This plot shows the partially corrected FGS and FGL data, and the angular offset.'
print,'Type .cont to see a histogram of the offset values.'
stop

plot,hist
print,'This is the histogram of the offset angles.

print,'Type .cont to update the spin model, and reload MOM and FGS for before/after comparison.'
stop

; Adjust the eclipse spin model on the fly with the correction we've just
; calculated.

smp=spinmodel_get_ptr(probe,use_eclipse_corrections=1)
smp->adjust_delta_phi,trange=eclipse_trange,delta_phi_offset=final_offset

; Reload FGS data to get a before/after comparison

thm_load_fit,probe=probe,trange=load_trange,level=1,type='calibrated',datatype='fgs',suffix='_after',true_dsl=1,use_eclipse_corrections=1

plot_names=[plot_names,'*fgs_after']

; Reload MOM velocity data to get a before/after comparison

thm_load_mom,probe=probe,trange=load_trange,level=1,type='calibrated',true_dsl=1,use_eclipse_corrections=1,suffix='_after'
;mom_vel_names=tnames('*velocity_after',mom_vel_count)
;if (mom_vel_count GT 0) then begin
;   plot_names=[plot_names,mom_vel_names]
;end

print,'Available tplot variables...before correction:'
tplot_names,'*before'
print, 'Available tplot variables...after correction:'
tplot_names,'*after'

print,'The above FGS and MOM variables can now be plotted to see a before/after comparison.'

print,'When you are finished, type .cont to proceed to the next eclipse.'
stop

;tplot,[fgs_name,fgs_name+'_after',fgl_name,'fgl_fit_offset'],trange=load_trange
;tplot,plot_names,trange=load_trange
;stop

end
